import time
import re

class EquipmentOperation:
    """
         设备操作
         Equipment operation
    """

    def start_app(self, name: str, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            启动 APP
            Start APP

            name: APP名字或者包名
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return:成功返回True 失败返回False

            name: APP name or package name
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: Returns True successfully, and returns False if it fails
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("startApp", name)
            if "false" in response or "null" in response:
                time.sleep(interval_time)
            else:
                return True
        return False

    def app_is_running(self, app_name: str) -> bool:
        """
            判断app是否正在运行(无障碍权限开启只判断前台，未开启则包含前后台 是否正在运行)
            Judge whether the app is running (if the barrier-free permission is enabled, only judge whether the foreground is running; if it is not enabled, it includes whether the foreground and background are running)

            return: 成功返回True 失败返回False
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("appIsRunnig", app_name) 

    def get_installed_packages(self) -> list:
        """
            获取已安装app的包名(不包含系统APP)
            Gets the package name of the installed APP (excluding the system app)

            return: 包名列表 或者 空[]
            return: The package name list is either empty []
        """
        resp = self.SendData("getInstalledPackages")
        if resp == "null" or resp == "":
            return []
        return resp.split("|")

    def get_title(self) -> str:
        """
            获取投屏标题
            Get the title of the projection screen

            return: app上的标题信息
            return: Title information on the app
        """
        response = self.SendData("getTitle")
        return response

    def get_group(self) -> str:
        """
            获取投屏组号
            Get the projection group number

            return: app上的投屏组号
            return: Projection group number on the app
        """
        response = self.SendData("getGroup")
        return response

    def get_identifier(self) -> str:
        """
            获取投屏编号
            Get the projection number

            return: app上的投屏编号
            return: Projection number on the app
        """
        response = self.SendData("getIdentifier")
        return response

    def activate_frame(self, secret_key) -> str:
        """
            激活安卓框架
            Activate Android framework

            secret_key：激活框架的秘钥
            secret_key：Key to activate the frame

            return: 成功返回True失败返回False
            return: Returns True successfully and False if it fails
        """
        response = self.SendData("activateFrame", secret_key) 
        return response

    def get_device_ip(self) -> str:
        """
            获取设备IP地址
            Get the device IP address

            return: 设备IP地址字符串
            return: device IP address string
        """
        return self.client_address[0]

    def get_android_id(self) -> str:
        """
            获取 Android 设备 ID
            Get AndroID device id

            return: Android 设备 ID 字符串
            return: AndroID device id string
        """
        response = self.SendData("getAndroidId")
        return response

    def get_window_size(self) -> dict:
        """
            获取屏幕大小
            Get the screen size

            return: 屏幕大小, 字典格式
            return: screen size, dictionary format
        """
        response = self.SendData("getWindowSize")
        width, height = response.split("|")
        return {"width": float(width), "height": float(height)}

    def get_image_size(self, image_path: str) -> dict:
        """
            获取图片大小
            Get the picture size

            image_path: 图片路径
            return: 图片大小, 字典格式, 失败返回 {'width': -1.0, 'height': -1.0}

            image_path: image path
            return: picture size, dictionary format
        """
        if not image_path.startswith("/storage/emulated/0/"):
            image_path = "/storage/emulated/0/" + image_path
        response = self.SendData("getImageSize", image_path)
        width, height = response.split("|")
        return {"width": float(width), "height": float(height)}

    def show_toast(self, text: str, duration: float = 3) -> bool:
        """
            Toast 弹窗
            Toast popup window

            text: 弹窗内容
            duration: 弹窗持续时间默认3秒
            return: 成功返回True 失败返回False

            text: pop-up contents
            duration: The default duration of pop-up window is 3 seconds
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("showToast", text, duration * 1000) 

    def send_keys(self, text: str) -> bool:
        """
            发送文本，需要打开 AiBot 输入法
            To send text, you need to open the AiBot input method

            text: 文本内容
            return:  成功返回True 失败返回False

            text: text content
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("sendKeys", text)  

    def send_vk(self, vk: int) -> bool:
        """
            发送 vk , 需要打开 AiBot 输入法
            Send vk

            vk: 虚拟键值
            return: 成功返回True 失败返回False

            虚拟键值按键对照表 https://blog.csdn.net/yaoyaozaiye/article/details/122826340
        """
        return "true" in self.SendData("sendVk", vk) 

    def back(self) -> bool:
        """
            返回
            return

            return: 成功返回True 失败返回False
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("back") 
    
    def home(self) -> bool:
        """
            返回桌面
            Back to the desktop

            return: 成功返回True 失败返回False
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("home") 

    def recent_tasks(self) -> bool:
        """
            显示最近任务
            Show recent tasks

            return: 成功返回True 失败返回False
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("recents")

    def power_dialog(self) -> bool:
        """
            打开 开/关机 对话框，基于无障碍权限
            Open the on/off dialog box, based on accessibility rights

            return:成功返回True 失败返回False
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("powerDialog") 

    def call_phone(self, mobile: str) -> bool:
        """
            拨打电话
            dial

            mobile: 手机号码
            return: 成功返回True 失败返回False

            mobile: mobile phone number
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("callPhone", mobile) 

    def send_msg(self, mobile, text) -> bool:
        """
            发送短信
            send a text message

            mobile: 手机号码
            text: 短信内容
            return: 成功返回True 失败返回False

            mobile: mobile phone number
            text: short message content
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("sendMsg", mobile, text)  

    def get_activity(self) -> str:
        """
            获取活动页
            Get active page

            return: 当前的窗口UI名称
            return: the current UI name of the window
        """
        response = self.SendData("getActivity")
        return response

    def get_package(self) -> str:
        """
            获取包名
            Get package name

            return: 包名称："com.aibot.client"
            return: package name: "com.aibot.client"
        """
        response = self.SendData("getPackage")
        return response

    def set_clipboard_text(self, text: str) -> bool:
        """
            设置剪切板文本
            Set clipboard text

            text: 文本
            return: 成功返回True 失败返回False

            text: text
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("setClipboardText", text)

    def get_clipboard_text(self) -> str:
        """
            获取剪切板内容（需要开启Aibote输入法）
            Get the clipboard content

            return: 剪切板内容
            return: Clipboard content
        """
        response = self.SendData("getClipboardText")
        return response

    def start_activity(self, action: str, uri: str = '', package_name: str = '', class_name: str = '', typ: str = '') -> bool:
        """
            Intent 跳转
            Intent jump

            action: 动作，例如 "android.intent.action.VIEW"
            uri: 跳转链接，例如：打开支付宝扫一扫界面，"alipayqr://platformapi/startapp?saId=10000007"
            package_name: 包名，"com.xxx.xxxxx"
            class_name: 类名
            typ: 类型
            return: True或者 False

            action: Action, such as "android.intent.action.VIEW"
            uri: jump link, for example: open Alipay scan interface, "AlipayQR://PlatformAPI/startapp? saId=10000007"
            package_name: package name, "com.xxx.xxxxx"
            class_name: class name
            typ: type
            return: True or False
        """
        return "true" in self.SendData("startActivity", action, uri, package_name, class_name, typ) 

    def write_android_file(self, remote_path: str, text: str, append: bool = False) -> bool:
        """
            写入安卓文件 
            Write to Android file

            remote_path: 安卓文件默认根目录:/storage/emulated/0/,  文件名必须是.txt后缀结
            text: 写入的内容
            append: 可选参数，是否追加，默认覆盖文件内容
            return: True 或者 False

            remote_path: Android file path /storage/emulated/0/: Android file root directory
            text: the text content to be written
            append: Whether to append the pattern
            return: True or False
        """
        if not remote_path.endswith(".txt"):
            raise TypeError("文件必须是.txt后缀结尾")

        if not remote_path.startswith("/storage/emulated/0/"):
            remote_path = "/storage/emulated/0/" + remote_path
        return "true" in self.SendData("writeAndroidFile", remote_path, text, append)

    def read_android_file(self, remote_path: str) -> str:
        """
            读取安卓文件 
            Read Android file

            remote_path: 安卓文件路径 /storage/emulated/0/: 安卓文件根目录
            return: 文件内容

            remote_path: Android file path /storage/emulated/0/: Android file root directory
            return: file content
        """
        if not remote_path.startswith("/storage/emulated/0/"):
            remote_path = "/storage/emulated/0/" + remote_path

        response = self.SendData("readAndroidFile", remote_path)
        if response == "null":
            return None
        return response

    def exists_android_file(self, remote_path: str) -> bool:
        """
            安卓文件是否存在
            Does the Android file exist

            remote_path: 安卓文件路径
            return: 存在返回True 不存在返回False

            Remote_path: Android file path
            Return: existence returns True; non-existence returns False
        """
        if not remote_path.startswith("/storage/emulated/0/"):
            remote_path = "/storage/emulated/0/" + remote_path

        return "true" in self.SendData("existsAndroidFile", remote_path)

    def get_android_sub_files(self, android_directory: str = "") -> list:
        """
            获取文件夹内的所有文件(不包含深层子目录)
            Get all files in the folder (excluding deep subdirectories)

            android_directory: 安卓目录,默认为根目录下所有文件名
            return: 文件名列表

            android_directory: Android directory, which defaults to all file names in the root directory
            return: list of file names
        """
        if not android_directory.startswith("/storage/emulated/0/"):
            android_directory = "/storage/emulated/0/" + android_directory

        response = self.SendData("getAndroidSubFiles", android_directory)
        if response == "null" or response == "":
            return []
        return response.split("|")

    def make_android_dir(self, android_directory: str) -> bool:
        """
            创建安卓文件夹
            Create Android folder

            android_directory: 安卓目录 /storage/emulated/0/: 安卓文件根目录
            return: 成功返回True 失败返回False

            android_directory: Android directory  /storage/emulated/0/: Android file root directory
            return: existence returns True; non-existence returns False
        """
        if not android_directory.startswith("/storage/emulated/0/"):
            android_directory = "/storage/emulated/0/" + android_directory
        return "true" in self.SendData("makeAndroidDir", android_directory) 

    def delete_android_file(self, remote_path: str) -> bool:
        """
            删除安卓文件
            Delete Android files

            remote_path: 安卓文件路径
            return: 成功返回True 失败返回False

            remote_path: Android file path.
            return: Returns True successfully, and returns False if it fails
        """
        if not remote_path.startswith("/storage/emulated/0/"):
            remote_path = "/storage/emulated/0/" + remote_path
        return "true" in self.SendData("deleteAndroidFile", remote_path) 

    def coordinate_transform(self, coordinate: tuple, resolution_a: tuple, resolution_b: tuple) -> tuple:
        """
            根据屏幕分辨率缩放比例，进行不同分辨率坐标转换
            According to the screen resolution scaling, coordinate conversion with different resolutions is carried out.

            coordinate: 需要装换的坐标可以是x,y坐标，也可以是矩形坐标 x1, y1, x2, y2
            resolution_a: 你抓取过坐标的设备标准分辨率
            resolution_b：你需要转换的设备分辨率
            retrun: 元祖

            coordinate: the coordinates to be replaced can be x,y coordinates or rectangular coordinates x1, y1, x2, y2
            resolution_a: The standard resolution of the device for which you have grabbed the coordinates
            resolution_b: The device resolution you need to convert
            retrun: Yuanzu
        """
        
        def convert_coordinates(x, y, resolution_a, resolution_b):
            W_A, H_A = resolution_a  
            W_B, H_B = resolution_b 
            

            scale_x = W_B / W_A
            scale_y = H_B / H_A
            

            x_b = x * scale_x
            y_b = y * scale_y
            
            return (x_b, y_b)

        if len(coordinate) == 2:
            result = convert_coordinates(coordinate[0], coordinate[1], resolution_a, resolution_b)
            return result

        if len(coordinate) == 4:
            result = convert_coordinates(coordinate[0], coordinate[1], resolution_a, resolution_b)
            result2 = convert_coordinates(coordinate[2], coordinate[3], resolution_a, resolution_b)
            return (result[0], result[1], result2[0], result2[1])
        return ()

    def close_driver(self):
        """
            关闭连接
            Close the connection
        """
        self.SendData("closeDriver")

    def set_android_timeout(self, timeout: float = 60):
        """
            设置安卓客户端接收超时，默认为永久等待
            Set the receiving timeout of Android client, and the default is permanent waiting

            timeout: 超时时间，单位秒
            return: 总是True

            timeout: timeout in milliseconds
            return: always True
        """
        self.SendData("setAndroidTimeout", timeout*1000)




class DrivingOperation:
    """
        驱动程序相关
        Driver correlation
    """

    def close_driver(self) -> bool:
        """
            关闭WebDriver.exe驱动程序，服务端会抛错
            Close the WebDriver.exe driver.

            return: 布尔值
            return: bool
        """
        return "true" in self.SendData("closeDriver") 


    def get_extend_param(self) -> str:
        """
            获取驱动程序命令行参数(不包含ip和port)
            Get the driver command line parameters (excluding ip and port)

            return: 成功返回参数，失败返回None
            return: parameter is returned successfully, and None is returned on failure.
        """

        response = self.SendData("getExtendParam")
        if response == "null":
            return None
        return response


    def activate_frame(self, secret_key) -> str:
        """
            激活Web框架
            Activate Web framework

            secret_key：激活框架的秘钥
            secret_key：Key to activate the frame

            return: 成功返回True失败返回False
            return: Returns True successfully and False if it fails
        """

        response = self.SendData("activateFrame", secret_key) 
        return response
    
    
    def set_download_dir(self, dir_name) -> str:
        """
            设置浏览器默认下载目录
            Set the browser default download directory

            dir_name：下载目录
            dir_name: download directory

            return: 成功返回True失败返回False
            return: Returns True successfully and False if it fails
        """

        return "true" in self.SendData("setDownloadDir", dir_name)



import re
import json

class ExcelOperation:
    """
        EXCEL操作
        EXCEL operation
    """

    def open_excel(self, excel_path: str) -> dict:
        """
            获取Excel对象
            Get Excel objects

            excel_path: excle路径
            return: excel对象或者None

            excel_path: excle path
            return: excel object or None
        """
        response = self.SendData("openExcel", excel_path)
        if response == "null":
            return None
        return json.loads(response)

    def open_excel_sheet(self, excel_object: object, sheet_name: str) -> str:
        """
            打开excel表格
            Open excel table

            excel_object: excel对象
            sheet_name: 表名
            return: 成功返回sheet对象，失败返回None

            Excel_object: excel object
            Sheet_name: table name
            Return: the sheet object is returned successfully, and None is returned on failure
        """
        response = self.SendData("openExcelSheet", excel_object['book'], excel_object['path'], sheet_name)
        if response == "null":
            return None
        return response

    def save_excel(self, excel_object: object) -> bool:
        """
            保存excel文档
            Save excel document

            excel_object: excel对象
            return: True或者False

            excel_object: excel object
            return: True or False
        """
        return "true" in self.SendData("saveExcel", excel_object['book'], excel_object['path']) 

    def write_excel_num(self, excel_object: object, row: int, col: int, value: int) -> bool:
        """
            写入数字到excel表格
            Write numbers to excel tables

            excel_object: excel对象
            row: 行
            col: 列
            value: 写入的值
            return: True或者False

            excel_object: excel object
            row: row
            col: column
            value: the value written
            return: True or Fals
        """
        return "true" in self.SendData("writeExcelNum", excel_object, col, row, value) 

    def write_excel_str(self, excel_object: object, row: int, col: int, str_value: str) -> bool:
        """
            写入字符串到excel表格
            Write a string to an excel table

            excel_object: excel对象
            row: 行
            col: 列
            str_value: 写入的值
            return: True或者False

            excel_object: Excel object
            row: row
            col: col
            str_value: Written value
            return: True or False
        """
        return "true" in self.SendData("writeExcelStr", excel_object, row, col, str_value)

    def read_excel_num(self, excel_object: object, row: int, col: int) -> float:
        """
            读取excel表格数字
            Read excel table numbers

            excel_object: excel对象
            row: 行
            col: 列
            return: 读取的数字

            excel_object: Excel object
            row: row
            col: col
            return: The number read
        """
        response = self.SendData("readExcelNum", excel_object, col, row)
        return float(response)

    def read_excel_str(self, excel_object: object, row: int, col: int) -> str:
        """
            读取excel表格字符串
            Read excel table string

            excel_object: excel对象
            row: 行
            col: 列
            return: 读取的字符串

            excel_object: Excel object
            row: row
            col: col
            return: Read character
        """
        response = self.SendData("readExcelStr", excel_object, row, col)
        return response

    def remove_excel_row(self, excel_object: object, row_first: int, row_last: int) -> bool:
        """
            删除excel表格行
            Delete excel table rows

            excel_object: excel对象
            row_first: 起始行
            row_last: 结束行
            return: True或者False

            excel_object: Excel object
            row_first: Starting line
            row_last: End line
            return: True or False
        """
        return "true" in self.SendData("removeExcelRow", excel_object, row_first, row_last)

    def remove_excel_col(self, excel_object: object, col_first: int, col_last: int) -> bool:
        """
            删除excel表格列
            Delete excel table columns

            excel_object: excel对象
            col_first: 起始列
            col_last: 结束列
            return: True或者False

            excel_object: Excel object
            col_first: Starting column
            col_last: End column
            return: True or False 
        """
        return "true" in self.SendData("removeExcelCol", excel_object, col_first, col_last) 






class KeymouseOperation:
    """
        键鼠操作
        Keymouse operation
    """

    def move_mouse(self, hwnd: str, x: float, y: float, mode: bool = False, ele_hwnd: str = "0") -> bool:
        """
            移动鼠标

            hwnd: 当前窗口句柄
            x: 横坐标
            y: 纵坐标
            mode: 操作模式，后台 True，前台 False, 默认前台操作
            ele_hwnd: 元素句柄，如果 mode=True 且目标控件有单独的句柄，则需要通过 get_element_window 获得元素句柄，指定 ele_hwnd 的值(极少应用窗口由父窗口响应消息，则无需指定)
            return: 总是返回True

            hwnd: Current window handle
            x: abscissa
            y: ordinate
            mode: Operation mode, background True, foreground False, default foreground operation
            ele_hwnd: Element handle, if mode=True and the target control has a separate handle, you need to get the element handle through get_element_window, and specify the value of ele_hwnd (it is unnecessary to specify if the application window is rarely responded to by the parent window)
            return: Always returns True
        """
        return "true" in self.SendData("moveMouse", hwnd, x, y, mode, ele_hwnd)

    def move_mouse_relative(self, hwnd: str, x: float, y: float, mode: bool = False) -> bool:
        """
            移动鼠标(相对坐标)
            Move the mouse (relative coordinates)

            hwnd: 当前窗口句柄
            x: 相对横坐标
            y: 相对纵坐标
            mode: 操作模式，后台 True，前台 False, 默认前台操作
            return: 总是返回True

            hwnd: Current window handle
            x: Relative abscissa
            y: Relative ordinate
            mode: Operation mode, background True, foreground False, default foreground operation
            return: Always returns True
        """
        return "true" in self.SendData("moveMouseRelative", hwnd, x, y, mode)

    def scroll_mouse(self, hwnd: str, x: float, y: float, count: int, mode: bool = False) -> bool:
        """
            滚动鼠标
            Scroll mouse

            hwnd: 当前窗口句柄
            x: 横坐标
            y: 纵坐标
            count: 鼠标滚动次数, 负数下滚鼠标, 正数上滚鼠标
            mode: 操作模式，后台 True，前台 False, 默认前台操作
            return: 总是返回True

            hwnd: Current window handle
            x: abscissa
            y: ordinate
            count: The number of mouse scrolling, negative scrolling down, positive scrolling up
            mode: Operation mode, background True, foreground False, default foreground operation
            return: Always returns True
        """
        return "true" in self.SendData("rollMouse", hwnd, x, y, count, mode)

    def click_mouse(self, hwnd: str, x: float, y: float, typ: int, mode: bool = False, ele_hwnd: str = "0") -> bool:
        """
            鼠标点击
            Mouse click

            hwnd: 当前窗口句柄
            x: 横坐标
            y: 纵坐标
            typ: 点击类型，单击左键:1 单击右键:2 按下左键:3 弹起左键:4 按下右键:5 弹起右键:6 双击左键:7 双击右键:8
            mode: 操作模式，后台 true，前台 false, 默认前台操作
            ele_hwnd: 元素句柄，如果 mode=True 且目标控件有单独的句柄，则需要通过 get_element_window 获得元素句柄，指定 ele_hwnd 的值(极少应用窗口由父窗口响应消息，则无需指定);
            return: 总是返回True

            hwnd: Current window handle
            x: abscissa
            y: ordinate
            typ: Click type, click left key: 1 Right key: 2 Press left key: 3 Pop up left key: 4 Press right key: 5 Pop up right key: 6 Double click left key: 7 Double click right key: 8
            mode: Operation mode, background True, foreground False, default foreground operation
            ele_hwnd: Element handle, if mode=True and the target control has a separate handle, you need to get the element handle through get_element_window, and specify the value of ele_hwnd (it is unnecessary to specify if the application window is rarely responded to by the parent window)
            return: Always returns True
        """
        return "true" in self.SendData("clickMouse", hwnd, x, y, typ, mode, ele_hwnd) 

    def send_keys(self, text: str) -> bool:
        """
            输入文本
            Input text

            text: 输入的文本
            return: 总是返回True

            text: Entered text
            return: Always returns True
        """
        return "true" in self.SendData("sendKeys", text) 

    def send_keys_by_hwnd(self, hwnd: str, text: str) -> bool:
        """
            后台输入文本(杀毒软件可能会拦截)
            Enter text in the background (antivirus software may intercept it)

            hwnd: 窗口句柄
            text: 输入的文本
            return: 总是返回True

            hwnd: Window handle
            text: Entered text
            return: Always returns True
        """
        return "true" in self.SendData("sendKeysByHwnd", hwnd, text)

    def send_vk(self, vk: int, typ: int) -> bool:
        """
            输入虚拟键值(VK)
            Enter the virtual key value (VK)

            vk: VK键值  按键对照表: https://blog.csdn.net/qq_42372031/article/details/105178789
            typ: 输入类型，按下弹起:1 按下:2 弹起:3
            return: 总是返回True

            vk: VK key value key table: https://blog.csdn.net/qq_42372031/article/details/105178789
            typ: Enter the type, press and pop up: 1 press: 2 pop up: 3
            return: Always returns True
        """
        return "true" in self.SendData("sendVk", vk, typ)

    def send_vk_by_hwnd(self, hwnd: str, vk: int, typ: int) -> bool:
        """
            后台输入虚拟键值(VK)
            Background input virtual key value (VK)

            hwnd: 窗口句柄
            vk: VK键值
            typ: 输入类型，按下弹起:1 按下:2 弹起:3
            return: 总是返回True

            hwnd: Window handle
            vk: VK key value
            typ: Enter the type, press and pop up: 1 press: 2 pop up: 3
            return: Always returns True
        """
        return "true" in self.SendData("sendVkByHwnd", hwnd, vk, typ)







import re


class VoiceService:
    """
        语音服务
        Language service
    """

    def init_speech_service(self, speech_key: str, speech_region: str) -> bool:
        """
            初始化语音服务(不支持win7)，需要调用 activateSpeechService 激活
            To initialize voice service (win7 is not supported), you need to call activateSpeechService to activate it

            speech_key: 微软语音API密钥
            speech_region: 区域
            return: True或者False

            speech_key: Microsoft voice API key
            speech_region: region
            return: True or False
        """
        return "true" in self.SendData("initSpeechService", speech_key, speech_region) 

    def audio_file_to_text(self, file_path, language: str) -> str:
        """
            音频文件转文本
            Audio file to text

            file_path: 音频文件路径
            language: 语言，参考开发文档 语言和发音人 zh-cn
            return: 转换后的音频文本或者None

            file_path: the audio file path
            language: language, reference development document language and speaker zh-cn
            return: converted audio text or None
        """
        response = self.SendData("audioFileToText", file_path, language)
        if response == "null":
            return None
        return response

    def microphone_to_text(self, language: str) -> str:
        """
            麦克风输入流转换文本
            Microphone input stream conversion text

            language: 语言，参考开发文档 语言和发音人 zh-cn
            return: 转换后的音频文本或者None

            language: language, reference development document language and speaker zh-cn
            return: converted audio text or None
        """
        response = self.SendData("microphoneToText", language)
        if response == "null":
            return None
        return response

    def text_to_bullhorn(self, ssml_path_or_text: str, language: str, voice_name: str) -> bool:
        """
            文本合成音频到扬声器
            Text synthesis audio to speakers

            ssml_path_or_text: 要转换语音的文本或者".xml"格式文件路径
            language: 语言，参考开发文档 语言和发音人 zh-cn
            voice_name: 发音人，参考开发文档 语言和发音人  zh-cn-XiaochenNeural
            return: True或者False

            ssml_path_or_text: The path of the text or ".xml" format file to be converted into speech
            language: Language, reference development document language and speaker  zh-cn
            voice_name: Speaker, reference development document language and speaker zh-cn-XiaochenNeural
            return: True or False  
        """
        return "true" in self.SendData("textToBullhorn", ssml_path_or_text, language, voice_name) 

    def text_to_audio_file(self, ssml_path_or_text: str, language: str, voice_name: str, audio_path: str) -> bool:
        """
            文本合成音频并保存到文件
            Text synthesizes audio and saves it to a file.

            ssml_path_or_text: 要转换语音的文本或者".xml"格式文件路径
            language: 语言，参考开发文档 语言和发音人
            voice_name: 发音人，参考开发文档 语言和发音人
            audio_path: 保存音频文件路径
            return: True或者False

            ssml_path_or_text: the text or file path in ".xml" format to be converted
            language: language, reference development document language and speaker
            voice_name: speaker, refer to the development document language and speaker
            audio_path: the path to save the audio file
            return: True or False
        """
        return "true" in self.SendData("textToAudioFile", ssml_path_or_text, language, voice_name, audio_path) 

    def microphone_translation_text(self, source_language: str, target_language: str) -> str:
        """
            麦克风音频翻译成目标语言文本
            Microphone audio is translated into target language text

            source_language: 要翻译的语言，参考开发文档 语言和发音人
            target_language: 翻译后的语言，参考开发文档 语言和发音人
            return: 转换后的音频文本或者None

            source_language: the language to be translated, refer to the development document language and speaker
            target_language: the translated language, referring to the development document language and speaker
            return: converted audio text or None
        """
        response = self.SendData("microphoneTranslationText", source_language, target_language)
        if response == "null":
            return None
        return response

    def audio_file_translation_text(self, audio_path: str, source_language: str, target_language: str) -> str:
        """
            麦克风输入流转换文本
            Microphone input stream conversion text

            audio_path: 要翻译的音频文件路径
            source_language: 要翻译的语言，参考开发文档 语言和发音人
            target_language: 翻译后的语言，参考开发文档 语言和发音人
            return: 转换后的音频文本或者None

            audio_path: the path of the audio file to be translated
            source_language: the language to be translated, refer to the development document language and speaker
            target_language: the translated language, referring to the development document language and speaker
            return: converted audio text or None
        """
        response = self.SendData("audioFileTranslationText", audio_path, source_language, target_language)
        if response == "null":
            return None
        return response



from .DebugStartDriver import  Driver
from .SocketServer import  ThreadingTCPServer
from .Logger import  LoggerRecord
from .UniversalFunction import  UniversalFunction
from .SendTcpData import  SendClientData
from .DatabaseFunc import DataBaseHandle
from .WriteReadFileFunc import WriteReadFile
from .DatabaseFunc import DataBaseHandle
from .Sqlite3DataBase import Sqlite3DataBaseHandle
from .WebsocketsServer import WebSocketServerThread, WebSocketServerUse

__all__ = ["Driver","ThreadingTCPServer","LoggerRecord","UniversalFunction","SendClientData","DataBaseHandle","WriteReadFile","Sqlite3DataBaseHandle", "WebSocketServerThread", "WebSocketServerUse"]


import re,json


class VerificationCodeOperation:
    """
        验证码
        Verification Code
    """
    def get_captcha(self, file_path: str, username: str, password: str, soft_id: str, code_type: str, len_min: str = '0') -> dict:
        """
            识别验证码
            Identification verification code

            file_path: 图片文件路径
            username: 用户名
            password: 密码
            soft_id: 软件ID
            code_type: 图片类型 参考 https://www.chaojiying.com/price.html
            len_min: 最小位数 默认0为不启用,图片类型为可变位长时可启用这个参数
            return: JSON
                err_no,(数值) 返回代码  为0 表示正常，错误代码 参考 https://www.chaojiying.com/api-23.html
                err_str,(字符串) 中文描述的返回信息 
                pic_id,(字符串) 图片标识号，或图片id号
                pic_str,(字符串) 识别出的结果
                md5,(字符串) md5校验值,用来校验此条数据返回是否真实有效

            file_path: the path of the picture file
            username: user name
            password: password
            soft_id: software id
            code_type: the picture type refers to https://www.chaojiying.com/price.html
            len_min: The minimum number of digits is not enabled by default, and this parameter can be enabled when the picture type is variable bit length
            return: JSON
                err_no, (numerical value) The return code is 0, which means normal, and the error code refers to https://www.chaojiying.com/api-23.html
                err_str, (string) the return information described in Chinese
                pic_id, (string) picture identification number, or picture ID number
                pic_str, (string) the result of recognition
                md5, (string) md5 check value, which is used to check whether this data return is true and valid
        """
        if not file_path.startswith("/storage/emulated/0/"):
            file_path = "/storage/emulated/0/" + file_path

        response = self.SendData("getCaptcha", file_path, username, password, soft_id, code_type, len_min)
        return json.loads(response)

    def error_captcha(self, username: str, password: str, soft_id: str, pic_id: str) -> dict:
        """
            识别报错返分
            Identify and report errors and return points

            username: 用户名
            password: 密码
            soft_id: 软件ID
            pic_id: 图片ID 对应 getCaptcha返回值的pic_id 字段
            return: JSON
                err_no,(数值) 返回代码
                err_str,(字符串) 中文描述的返回信息

            username: user name
            password: password
            soft_id: software id
            pic_id: the picture id corresponds to the pic_id field of getCaptcha return value
            return: JSON
                err_no, (numeric) return code
                err_str, (string) the return information described in Chinese
        """
        response = self.SendData("errorCaptcha", username, password, soft_id, pic_id)
        return json.loads(response)

    def score_captcha(self, username: str, password: str) -> dict:
        """
            查询验证码剩余题分
            Query the remaining questions of verification code

            username: 用户名
            password: 密码
            return: JSON
                err_no,(数值) 返回代码
                err_str,(字符串) 中文描述的返回信息
                tifen,(数值) 题分
                tifen_lock,(数值) 锁定题分

            username: user name
            password: password
            return: JSON
                err_no, (numeric) return code
                err_str, (string) the return information described in Chinese
                tifen, (numerical) score
                tifen_lock, (numerical value) locks the score
        """
        response = self.SendData("scoreCaptcha", username, password)
        return json.loads(response)





import time

class CoordinateOperation:
    """
        坐标操作
        Coordinate operation
    """

    def click(self, point: tuple) -> bool:
        """
            点击坐标
            Click coordinates

            point: x, y坐标
            return: Ture 或者 False

            point: coordinates
            return: Ture or False
        """
        return "true" in self.SendData("click", point[0], point[1])

    def double_click(self, point: tuple) -> bool:
        """
            双击坐标
            Double-click coordinates

            point: 坐标
            return: Ture 或者 False

            point: coordinates
            return: Ture or False
        """
        return "true" in self.SendData("doubleClick", point[0], point[1]) 

    def long_click(self, point: tuple, duration: int) -> bool:
        """
            长按坐标
            Long press coordinate

            point: 坐标
            duration: 按住时长，单位秒
            return: Ture 或者 False

            point: coordinates
            duration: press and hold time, in seconds
            return: Ture or False
        """
        return "true" in self.SendData("longClick", point[0], point[1], duration * 1000) 

    def swipe(self, start_point: tuple, end_point: tuple, duration: float) -> bool:
        """
            滑动坐标
            Sliding coordinate

            start_point: 起始坐标
            end_point: 结束坐标
            duration: 滑动时长，单位秒
            return: Ture 或者 False

            start_point: the starting coordinate
            end_point: end coordinate
            duration: sliding duration, in seconds
            return: Ture or False
        """
        return "true" in self.SendData("swipe", start_point[0], start_point[1], end_point[0], end_point[1], duration * 1000)

    def gesture(self, gesture_path: list, duration: float) -> bool:
        """
            执行手势
            Execute a gesture

            gesture_path: 手势路径，由一系列坐标点组成
            duration: 手势执行时长, 单位秒
            return: Ture 或者 False

            gesture_path: gesture path, which consists of a series of coordinate points
            duration: duration of gesture execution, in seconds
            return: Ture or False
        """

        gesture_path_str = ""
        for point in gesture_path:
            gesture_path_str += f"{point[0]}/{point[1]}/\n"
        gesture_path_str = gesture_path_str.strip()

        return "true" in self.SendData("dispatchGesture", gesture_path_str, duration * 1000) 

    def gestures(self, gestures_path: list) -> bool:
        """
            执行多个手势
            Perform multiple gestures

            [[duration, [x1, y1], [x1, y1]...],[duration, [x1, y1], [x1, y1]...]]  
            duration:手势执行时长, 单位秒
            [x1, y1]: 手势路径，由一系列坐标点组成
            return: Ture 或者 False

            [[duration, [x1, y1], [x1, y1]...],[duration, [x1, y1], [x1, y1]...]]
            duration: duration of gesture execution, in seconds.
            [x1, y1]: gesture path, which consists of a series of coordinate points.
            return: Ture or False
        """

        gestures_path_str = ""
        for gesture_path in gestures_path:
            gestures_path_str += f"{gesture_path[0] * 1000}/"
            for point in gesture_path[1:len(gesture_path)]:
                gestures_path_str += f"{point[0]}/{point[1]}/\n"
            gestures_path_str += "\r\n"
        gestures_path_str = gestures_path_str.strip()

        return "true" in self.SendData("dispatchGestures", gestures_path_str) 

    def press(self, point: tuple, duration: float) -> bool:
        """
            手指按下
            Finger press

            point: 坐标
            duration: 持续时间，单位秒
            return: Ture 或者 False

            point: coordinates
            duration: Duration, in seconds
            return: Ture or False
        """
        return "true" in self.SendData("press", point[0], point[1], duration * 1000) 

    def move(self, point: tuple, duration: float) -> bool:
        """
            手指移动
            Finger movement

            point: 坐标
            duration: 持续时间
            return: Ture 或者 False

            point: coordinates
            duration: duration
            return: Ture or False
        """
        return "true" in self.SendData("move", point[0], point[1], duration * 1000) 

    def release(self) -> bool:
        """
            手指抬起
            Finger lift
        """
        return "true" in self.SendData("release") 

    def press_release(self, point: tuple, duration: float) -> bool:
        """
            按下屏幕坐标点并释放
            Press the screen coordinate point and release it

            point: 按压坐标
            duration: 按压时长，单位秒
            return: Ture 或者 False

            point: press coordinates
            duration: duration of pressing, in seconds
            return: Ture or False
        """
        result = self.press(point, duration)
        if not result:
            return False
        time.sleep(duration)
        result2 = self.release()
        if not result2:
            return False
        return True




import re

class ScreenshotOperation:
    """
        截图操作
        Screenshot operation
    """
    def __pull_file(self, *args) -> bytes:
        args_len = ""
        args_text = ""

        for argv in args:
            argv = str(argv)
            args_text += argv
            args_len += str(len(bytes(argv, 'utf8'))) + "/"
        data = (args_len.strip("/") + "\n" + args_text).encode("utf8")
        if len(data) > 10000:
            self.debug(rf"<-<- {data[:100]}......")
        else:
            self.debug(rf"<-<- {data}")
        self.request.sendall(data)
        response = self.request.recv(65535)
        if len(response) > 10000:
            self.debug(rf"<-<- {response[:100]}......")
        else:
            self.debug(rf"<-<- {response}")
        if response == b"":
            self.request.close()
            raise ConnectionAbortedError(f"{self.client_address[0]}:{self.client_address[1]} 客户端断开链接")
        data_length, data = response.split(b"/", 1)
        while int(data_length) > len(data):
            data += self.request.recv(65535)
        return data

    def save_screenshot(self, image_name: str, region: tuple = (0,0,0,0), algorithm: tuple = (0,0,0)) -> str:
        """
            保存截图，返回图片地址(手机中)或者 None
            Save the screenshot and return to the picture address (in the phone) or None

            image_name: 图片名称，保存在手机 /storage/emulated/0/ 路径下；
            region: 截图区域，默认全屏，``region = (起点x、起点y、终点x、终点y)``，得到一个矩形
            algorithm: 处理截图所用算法和参数，默认保存原图，
                处理屏幕截图所用的算法，默认原图，注意：给定图片处理时所用的算法，应该和此方法的算法一致；
                ``algorithm = (algorithm_type, threshold, max_val)``

                按元素顺序分别代表：
                0. ``algorithm_type`` 算法类型
                1. ``threshold`` 阈值
                2. ``max_val`` 最大值

                ``threshold`` 和 ``max_val`` 同为 255 时灰度处理.
                ``algorithm_type`` 算法类型说明:
                    0. ``THRESH_BINARY``      算法，当前点值大于阈值 `threshold` 时，取最大值 ``max_val``，否则设置为 0；
                    1. ``THRESH_BINARY_INV``  算法，当前点值大于阈值 `threshold` 时，设置为 0，否则设置为最大值 max_val；
                    2. ``THRESH_TOZERO``      算法，当前点值大于阈值 `threshold` 时，不改变，否则设置为 0；
                    3. ``THRESH_TOZERO_INV``  算法，当前点值大于阈值 ``threshold`` 时，设置为 0，否则不改变；
                    4. ``THRESH_TRUNC``       算法，当前点值大于阈值 ``threshold`` 时，设置为阈值 ``threshold``，否则不改变；
                    5. ``ADAPTIVE_THRESH_MEAN_C``      算法，自适应阈值；
                    6. ``ADAPTIVE_THRESH_GAUSSIAN_C``  算法，自适应阈值；
            return: 图片地址(手机中) 或者 None

            Image_name: the name of the picture, which is saved in the path of mobile phone /storage/emulated/0/
            Region: screenshot area, full screen by default, `` region = (starting point x, starting point y, ending point x, ending point y) ```, and a rectangle is obtained
            Algorithm: the algorithm and parameters used to process screenshots, and the original image is saved by default
            The algorithm used to process screen shots defaults to the original image. Note: the algorithm used in the given image processing should be consistent with the algorithm of this method
                ``algorithm = (algorithm_type, threshold, max_val)``

                Represents, respectively, in the order of elements:
                0. ``algorithm_type ``` algorithm type
                1. ``threshold ```` threshold
                2. `` Max _ val```` Maximum value

                Gray processing when threshold and max_val are both 255.
                `` algorithm _ type``` description of algorithm_type:
                0. ``` threshold _ binary``` algorithm, when the current point value is greater than the threshold value ` threshold `, the maximum value ``` max _ val```` is taken, otherwise it is set to 0;
                1. `` threshold _ binary _ inv``` algorithm, when the current point value is greater than the threshold value ` threshold `, set it to 0, otherwise set it to the maximum value max_val；;
                2. `` threshold _ tozero``` algorithm, when the current point value is greater than the threshold ` threshold', it will not be changed, otherwise it will be set to 0;
                3. `` threshold _ tozero _ inv``` algorithm, when the current point value is greater than the threshold value ``` threshold ```, it is set to 0, otherwise it will not change;
                4. `` threshold _ trunc``` algorithm, when the current point value is greater than the threshold value ``` threshold ```, it is set as the threshold value ``` threshold ```, otherwise it will not be changed;
                5. ``` Adaptive _ threshold _ mean _ c``` algorithm, adaptive threshold;
                6. ``` Adaptive _ Thresh _ Gaussian _ C``` algorithm, adaptive threshold;
            Return: picture address (in mobile phone) or None.
        """
        if not image_name.startswith("/storage/emulated/0/"):
            image_name = "/storage/emulated/0/" + image_name

        algorithm_type, threshold, max_val = algorithm
        if algorithm_type in (5, 6):
            threshold = 127
            max_val = 255

        response = self.SendData("saveScreenshot", image_name, *region, algorithm_type, threshold, max_val)
        if "true" in response:
            return image_name
        return None

    def save_element_screenshot(self, image_name: str, xpath: str) -> str:
        """
            保存元素截图
            Save element screenshot

            image_name: 图片名称，保存在手机 /storage/emulated/0/ 路径下
            xpath: xpath路径
            return: 图片地址(手机中)或者 None

            image_name: the name of the picture, which is saved in the path of mobile phone /storage/emulated/0/
            xpath: xpath path
            return: picture address (in mobile phone) or None
        """
        response = self.SendData("getElementRect", xpath)
        if response == "-1|-1|-1|-1":
             return None
        start_x, start_y, end_x, end_y = response.split("|")
        return self.save_screenshot(image_name, region=(start_x, start_y, end_x, end_y))

    def take_screenshot(self, region: tuple = (0,0,0,0), algorithm: tuple = (0,0,0), scale: float = 1.0) -> bytes:
        """
            保存截图，返回图像字节格式或者None, 此功能如果做投屏scale参数需要缩放3倍以上不然很慢

            region: 截图区域，默认全屏，``region = (起点x、起点y、终点x、终点y)``，得到一个矩形
            algorithm: 处理截图所用算法和参数，默认保存原图
                处理屏幕截图所用的算法，默认原图，注意：给定图片处理时所用的算法，应该和此方法的算法一致
                ``algorithm = (algorithm_type, threshold, max_val)``

                按元素顺序分别代表：
                0. ``algorithm_type`` 算法类型
                1. ``threshold`` 阈值
                2. ``max_val`` 最大值

                ``threshold`` 和 ``max_val`` 同为 255 时灰度处理
                ``algorithm_type`` 算法类型说明:
                    0. ``THRESH_BINARY``      算法，当前点值大于阈值 `threshold` 时，取最大值 ``max_val``，否则设置为 0
                    1. ``THRESH_BINARY_INV``  算法，当前点值大于阈值 `threshold` 时，设置为 0，否则设置为最大值 max_val
                    2. ``THRESH_TOZERO``      算法，当前点值大于阈值 `threshold` 时，不改变，否则设置为 0
                    3. ``THRESH_TOZERO_INV``  算法，当前点值大于阈值 ``threshold`` 时，设置为 0，否则不改变
                    4. ``THRESH_TRUNC``       算法，当前点值大于阈值 ``threshold`` 时，设置为阈值 ``threshold``，否则不改变
                    5. ``ADAPTIVE_THRESH_MEAN_C``      算法，自适应阈值；
                    6. ``ADAPTIVE_THRESH_GAUSSIAN_C``  算法，自适应阈值；

            scale: 图片缩放率，默认为 1.0，1.0 以下为缩小，1.0 以上为放大
            return: 图像字节格式或者"null"的字节格式
        """
        algorithm_type, threshold, max_val = algorithm
        if algorithm_type in (5, 6):
            threshold = 127
            max_val = 255

        response = self.__pull_file("takeScreenshot", *region, algorithm_type, threshold, max_val, scale)
        if b"null" in response:
            return None
        return response








import re


class WinHidCorrelation:
    """
        windows hid
        Windows hid correlation
    """

    def init_hid(self) -> bool:
        """
            初始化Hid
            Initialize Hid

            return: True或者False
            return: True or False
        """
        return "true" in self.SendData("initHid") 

    def get_hid_data(self) -> list:
        """
            获取Hid相关数据 先调用 windowsBot.initHid，再调用androidBot.initHid
            Call windowsbot. initHid before calling androidBot.initHid to obtain hid related data.

            return: 激活成功的hid手机的安卓ID
            return: Android ID of successfully activated hid mobile phone
        """
        response = self.SendData("getHidData")
        if response == "":
            return []
        return response.split("|")

    def hid_press(self, android_id: str, angle: int, x: float, y: float) -> bool:
        """
            按下
            press

            android_id: 安卓id
            angle: 手机旋转角度
            x: 横坐标
            y: 纵坐标
            return: True或者False

            android_id: Android id
            angle: the rotation angle of mobile phone
            x: abscissa
            y: ordinate
            return: True or False
        """
        return "void" in self.SendData("hidPress", android_id, angle, x, y) 

    def hid_release(self, android_id: str, angle: int) -> bool:
        """
            释放
            release

            android_id: 安卓id
            angle: 手机旋转角度
            return: True或者False

            android_id: Android id
            angle: the rotation angle of mobile phone
            return: True or False
        """
        return "void" in self.SendData("hidRelease", android_id, angle) 

    def hid_move(self, android_id: str, angle: int, x: float, y: float, duration: float) -> bool:
        """
            移动
            move

            android_id: 安卓id
            angle: 手机旋转角度
            x: 横坐标
            y: 纵坐标
            duration: 移动时长,秒
            return: True或者False

            android_id: Android id
            angle: the rotation angle of mobile phone
            x: abscissa
            y: ordinate
            duration: moving duration, seconds
            return: True or False
        """
        return  "void" in self.SendData("hidMove", android_id, angle, x, y, duration * 1000) 

    def hid_click(self, android_id: str, angle: int, x: float, y: float) -> bool:
        """
            单击
            click

            android_id: 安卓id
            angle: 手机旋转角度
            x: 横坐标
            y: 纵坐标
            return: True或者False

            android_id: Android id
            angle: the rotation angle of mobile phone
            x: abscissa
            y: ordinate
            return: True or False
        """
        return "void" in self.SendData("hidClick", android_id, angle, x, y) 

    def hid_double_click(self, android_id: str, angle: int, x: float, y: float) -> bool:
        """
            双击
            double click

            android_id: 安卓id
            angle: 手机旋转角度
            x: 横坐标
            y: 纵坐标
            return: True或者False

            android_id: Android id
            angle: the rotation angle of mobile phone
            x: abscissa
            y: ordinate
            return: True or False
        """
        return "void" in self.SendData("hidDoubleClick", android_id, angle, x, y) 

    def hid_long_click(self, android_id: str, angle: int, x: float, y: float, duration: float) -> bool:
        """
            长按
            Long press

            android_id: 安卓id
            angle: 手机旋转角度
            x: 横坐标
            y: 纵坐标
            duration: 按下时长,秒
            return: True或者False

            android_id: Android id
            angle: the rotation angle of mobile phone
            x: abscissa
            y: ordinate
            duration: press duration, seconds
            return: True or False
        """
        return "void" in self.SendData("hidLongClick", android_id, angle, x, y, duration * 1000) 

    def hid_swipe(self, android_id: str, angle: int, startX: float, startY: float, endX: float, endY: float, duration: float) -> bool:
        """
            滑动坐标
            Sliding coordinate

            android_id: 安卓id
            angle: 手机旋转角度
            startX: 起始横坐标
            startY: 起始纵坐标
            endX: 结束横坐标
            endY: 结束纵坐标
            duration: 滑动时长,秒
            return: True或者False

            android_id: Android id
            angle: the rotation angle of mobile phone
            startX: starting abscissa
            startY: starting ordinate
            endX: end abscissa
            endY: end ordinate
            duration: sliding duration, seconds
            return: True or False
        """
        return "void" in self.SendData("hidSwipe", android_id, angle, startX, startY, endX, endY, duration * 1000) 

    def hid_gesture(self, android_id: str, angle: int, gesture_path: list, duration: float) -> bool:
        """
            Hid手势
            Hid gesture

            android_id: 安卓id
            angle: 手机旋转角度
            gesture_path: 手势路径，由一系列坐标点组成
            duration: 手势执行时长, 单位秒
            return: True或者False

            android_id: Android id
            angle: the rotation angle of mobile phone
            gesture_path: gesture path, which consists of a series of coordinate points
            duration: duration of gesture execution, in seconds
            return: True or False
        """
        gesture_path_str = ""
        for point in gesture_path:
            gesture_path_str += f"{point[0]}/{point[1]}/\n"
        gesture_path_str = gesture_path_str.strip()
        return "void" in self.SendData("hidDispatchGesture", android_id, angle, gesture_path_str, duration * 1000) 

    def hid_back(self, android_id: str) -> bool:
        """
            返回
            return

            android_id: 安卓id
            return: True或者False

            android_id: Android id
            return: True or False
        """
        return "void" in self.SendData("hidBack", android_id) 

    def hid_home(self, android_id: str) -> bool:
        """
            返回桌面
            home

            android_id: 安卓id
            return: True或者False

            android_id: Android id
            return: True or False
        """
        return "void" in self.SendData("hidHome", android_id) 

    def hid_recents(self, android_id: str) -> bool:
        """
            最近应用列表
            List of recent applications

            android_id: 安卓id
            return: True或者False

            android_id: Android id
            return: True or False
        """
        return "void" in self.SendData("hidRecents", android_id) 



from .CookiesOperation import  CookiesOperation
from .WebLoadWait import  WebLoadWait
from .PagesAndNavigation import  PagesNavigation
from .ElementOperation import  ElementOperation
from .KeymouseOperation import  KeymouseOperation
from .PopUpWindow import  PopUpWindow
from .WindowOperation import  WindowOperation
from .DrivingOperation import  DrivingOperation
from .JSinjection import  JSinjection
from ..CommonUse.ChatGenerative import ChatGenerative
from .IframeOperation import IframeOperation

__all__ = ["CookiesOperation","WebLoadWait","PagesNavigation","ElementOperation","KeymouseOperation","PopUpWindow","WindowOperation","DrivingOperation","JSinjection","ChatGenerative","IframeOperation"]


import re


class SystemOperation:
    """
        系统操作
        system operation
    """

    def set_clipboard_text(self, text: str) -> bool:
        """
            设置剪切板内容
            Set the clipboard contents

            text: 要设置的内容
            return: 成功返回True 失败返回 False

            text: what to set
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("setClipboardText", text)

    def get_clipboard_text(self) -> str:
        """
            获取剪切板内容
            Get the clipboard content

            return: 剪切板内容
            return: Shear board content
        """
        response = self.SendData("getClipboardText")
        return  response

    def start_process(self, cmd: str, show_window=True, is_wait=False) -> bool:
        """
            启动指定程序
            Start the specified program

            cmd: 命令
            show_window: 是否显示窗口，默认显示
            is_wait: 是否等待程序结束， 默认不等待
            return: 成功返回True 失败返回 False

            cmd: command
            show_window: whether to display the window or not; it is displayed by default
            is_wait: whether to wait for the program to end; by default, it does not wait
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData("startProcess", cmd, show_window, is_wait)

    def execute_command(self, command: str, wait_timeout: int = 3) -> str:
        """
            执行cmd命令
            Execute cmd command

            command: cmd命令，不能含 "cmd"字串
            wait_timeout: 可选参数，等待结果返回超时，单位毫秒，默认3秒
            return: cmd执行结果

            Command: cmd command, which cannot contain "cmd" string.
            Wait_timeout: optional parameter, waiting for the result to return timeout, in milliseconds, the default is 300 milliseconds.
            Return: cmd execution result
        """
        return  self.SendData("executeCommand", command, wait_timeout*1000)





"""
    PyAibote安卓云端自动化框架，采用云端服务模式构建。由安卓手机客户端主动连接服务器/电脑服务端程序，
    服务端应采用多线程TCP协议和多个安卓设备通信。构架模式决定了自动化代码部署在云端，能有效防止脚本被第三方恶意破解。
    结合Aibote远程投屏技术，可直接连接远程用户设备做自动化机器人编写，完美解决远程自动化测试需求。独家图色定位算法，
    50毫秒以内的速度遍历查找2340*1080分辨率的设备

    PyAibote Android cloud automation framework is built with cloud service mode. The Android mobile phone client actively connects with the server/computer server program,
    The server should use multithreaded TCP protocol to communicate with multiple Android devices. The architecture mode determines that the automation code is deployed in the cloud, 
    which can effectively prevent scripts from being maliciously cracked by third parties.
    Combined with Aibote remote screen projection technology, it can be directly connected to remote user equipment for automatic robot programming, 
    which perfectly meets the requirements of remote automatic testing. Exclusive color location algorithm,
    Traverse within 50 milliseconds to find the equipment with 2340*1080 resolution.
"""


import os,sys
import time
from multiprocessing import Process
from abc import ABC, abstractmethod
import socketserver,socket
from .Tool import  *
from .AndroidBotModel import *


class AndroidBotMain(
        ABC,
        LoggerRecord,
        ThreadingTCPServer, 
        AndroidLoadWait,
        Driver,
        SendClientData,
        UniversalFunction,
        CoordinateOperation,
        ElementOperation,
        EquipmentOperation,
        ScreenProjectionOperation,
        FileTransfer,
        Control,
        OcrCorrelation,
        YoloService,
        UrlRequest,
        ColorFindingOperation,
        MapFindingOperation,
        ScreenshotOperation,
        AndroidHidCorrelation,
        VerificationCodeOperation,
        ChatGenerative,
        DataBaseHandle,
        Sqlite3DataBaseHandle,
        WebSocketServerUse
    ):

    @abstractmethod
    def script_main(self):
        pass
    
    def handle(self):
        self.info(f"<-<- Client connection at {self.client_address[0]}: {self.client_address[1]}")
        self.debug(f"<-<- Client connection at {self.client_address[0]}: {self.client_address[1]}")
        self.script_main()


    @classmethod
    def execute(self, IP: str, Port, Qt = None, WebsocketSwitch = False, WebsocketPort = 8888):
        if Qt:
            self.Qt = Qt
        
        if WebsocketSwitch:
            server = WebSocketServerThread(IP, WebsocketPort, self.Websocket_Log_Level)
            server.start()

        ThreadingTCPServer.StartThreadingTCPServer(self, IP, Port)


    @classmethod
    def StopSrver(self):
        try:
            self.server.shutdown()  # 停止接受新的连接
            self.server.socket.close()  # 关闭服务器套接字
        except Exception as e:
            pass


















import json
import subprocess,os
import random
import platform


class Driver:
    """
        DEBUG模式下自动启动驱动调试浏览器
        Automatically start the driver DEBUGging browser in debug mode.
    """

    @classmethod
    def WebDriverStart(self, Port ,Driver_Params) -> None:
        default_params = {
            "serverIp": "127.0.0.1",
            "serverPort": Port,
            "browserName": "chrome",
            "debugPort": 0,
            "userDataDir": f"./UserData{random.randint(100000, 999999)}",
            "browserPath": None,
            "argument": None,
        }
        if Driver_Params:
            default_params.update(Driver_Params)
        default_params = json.dumps(default_params, ensure_ascii=False)

        
        try:
            print("Debug Model Start WebDriver ...")
            subprocess.Popen(["WebDriver.exe", default_params])
            print("Start WebDriver Successful，Execute Script")
        except FileNotFoundError as e:
            err_msg = "\nStart local WebDriver.exe fail Exception elimination step：\n1. Check WebDriver.exe Path；\n2. WebDriver.exe Add to system environment variable?"
            self.error(f"{err_msg}: " + str(e))

    @classmethod
    def WindowsDriverStart(self, IP, Port) -> None:
        try:
            system_info = platform.system()
            if system_info == "Windows":
                DriverName = "WindowsDriver.exe"
                version_info = platform.version()
                major_version = int(version_info.split('.')[0])
                if major_version < 10:
                    DriverName = "WindowsDriver_win7.exe"
                print("Debug Model Start WinDriver ...")
                args = {
                            "Action": "Authentication",
                            "Data": {
                                "Token": "PyAibote",
                            }
                    }
                os.popen(f"{DriverName} 127.0.0.1 {Port} {args}")
                print("Start WinDriver Successful，Execute Script")
        except FileNotFoundError as e:
            err_msg = "\nStart local WinDriver.exe fail Exception elimination step：\n1. Check WebDriver.exe Path；\n2. WebDriver.exe Add to system environment variable?"
            self.error(f"{err_msg}: " + str(e))

    @classmethod
    def DigtalHumanDriverStart(self, IP, Port) -> None:
        try:
            system_info = platform.system()
            if system_info == "Windows":
                DriverName = "AiDriver.exe"
                print("Debug Model Start AiDriver ...")
                message = {
                        "Action": "Authentication",
                        "Data": {
                            "Token": "PyAibote",
                        }
                }
                os.popen(f"{DriverName} 127.0.0.1 {Port} {message}")
                print("Start AiDriver Successful，Execute Script")
        except FileNotFoundError as e:
            err_msg = "\nStart local WinDriver.exe fail Exception elimination step：\n1. Check WebDriver.exe Path；\n2. WebDriver.exe Add to system environment variable?"
            self.error(f"{err_msg}: " + str(e))



import re

class ScreenProjectionOperation:
    """
        投屏操作
        Screen projection operation
    """

    def get_group_id(self) -> str:
        """
            获取投屏组号
            Get the projection group number

            return: 组号
            return: group number
        """
        response = self.SendData("getGroup")
        return response

    def get_identifier(self) -> str:
        """
            获取投屏编号
            Get the projection number

            return: 编号
            return: number
        """
        response =  self.SendData("getIdentifier")
        return response

    def get_title(self) -> str:
        """
            获取投屏标题
            Get the title of the projection screen

            return: 标题
            return: title
        """
        response = self.SendData("getTitle")
        return response





import re


class FileTransfer:
    """
        文件传输
        File transfer
    """
    def __push_file(self, func_name: str, to_path: str, file: bytes) -> bool:
        func_name = bytes(func_name, "utf8")
        to_path = bytes(to_path, "utf8")
        str_data = ""
        str_data += str(len(func_name)) + "/"  # func_name 字节长度
        str_data += str(len(to_path)) + "/"  # to_path 字节长度
        str_data += str(len(file)) + "\n"  # file 字节长度
        bytes_data = bytes(str_data, "utf8")
        bytes_data += func_name
        bytes_data += to_path
        bytes_data += file
        self.request.sendall(bytes_data)
        data = self.request.recv(65535)
        return data.decode("utf8")

    def __pull_file(self, *args) -> bytes:
        args_len = ""
        args_text = ""

        for argv in args:
            argv = str(argv)
            args_text += argv
            args_len += str(len(bytes(argv, 'utf8'))) + "/"
        data = (args_len.strip("/") + "\n" + args_text).encode("utf8")
        self.request.sendall(data)
        response = self.request.recv(65535)
        if response == b"":
            self.request.close()
            raise ConnectionAbortedError(f"{self.client_address[0]}:{self.client_address[1]} 客户端断开链接")
        data_length, data = response.split(b"/", 1)
        while int(data_length) > len(data):
            data += self.request.recv(65535)
        return data

    def push_file(self, origin_path: str, to_path: str) -> bool:
        """
            将电脑文件传输到手机端
            Transfer computer files to the mobile phone

            origin_path: 源文件路径
            to_path: 安卓外部存储根目录 /storage/emulated/0/
            return: True或者False

            Origin_path: source file path
            To_path: Android external storage root directory /storage/emulated/0/
            Return: True or False
        """

        if not to_path.startswith("/storage/emulated/0/"):
            to_path = "/storage/emulated/0/" + to_path

        with open(origin_path, "rb") as r:
            data = r.read()
        return "true" in self.__push_file("pushFile", to_path, data)

    def pull_file(self, remote_path: str, local_path: str) -> bool:
        """
            将手机文件传输到电脑端
            Transfer mobile phone files to the computer.

            remote_path: 手机端文件路径
            local_path: 电脑本地文件存储路径
            return: 文件字节数据

            remote_path: the file path of the mobile phone
            local_path: the local file storage path of the computer
            return: file byte data
        """
        if not remote_path.startswith("/storage/emulated/0/"):
            remote_path = "/storage/emulated/0/" + remote_path
        response = self.__pull_file("pullFile", remote_path)
        if response == b"null":
            return False
        with open(local_path, "wb") as w:
            w.write(response)
        return True


"""
    PyAibote 轻量化实时数字人，模型大小仅20M。支持windows、Android、IOS、web、小程序 所有平台

    PyAibote Lightweight real-time digital human, the model size is only 20M. Support all platforms of windows, Android, IOS, web and applet
"""


import os,sys
import time
import subprocess
from abc import ABC, abstractmethod
import socketserver,socket
from .Tool import  *
from .DigitalHumanModel import  *
import threading



class HumanBotMain(
        ABC,
        LoggerRecord,
        SendClientData,
        ThreadingTCPServer, 
        DigitalHumanLoadWait,
        NewDigitalHumanOperation,
        WebSocketServerUse

   
    ):

    def __init__(self,*args):
        if len(args) ==1:
            address_info = socket.getaddrinfo(None, args[0], socket.AF_INET, socket.SOCK_STREAM)[0]
            family, socket_type, proto, _, socket_address = address_info
            server = socket.socket(family, socket_type, proto)
            server.bind(socket_address)
            server.listen(1)
            print("WindowsBot Service started successfully ...")
            self.request, self.client_address = server.accept()
            print("WindowsBot Client link succeeded")
        else:
            super().__init__(*args)


    # @abstractmethod
    def script_main(self):
        pass
    
    def handle(self):
        self.info(f"<-<- Client connection at {self.client_address[0]}: {self.client_address[1]}")
        self.debug(f"<-<- Client connection at {self.client_address[0]}: {self.client_address[1]}")
        self.script_main()

    @classmethod
    def execute(self, IP: str, Port: int, Debug: bool = True, Qt = None, WebsocketSwitch = False, WebsocketPort = 8888):
        try:
            if Port < 0 or Port > 65535:
                raise OSError("`listen_port` must be in 0-65535.")
            
            if Qt:
                self.Qt = Qt

            if Debug:
                Driver.DigtalHumanDriverStart(IP, Port)
                
            if WebsocketSwitch:
                server = WebSocketServerThread(IP, WebsocketPort, self.Websocket_Log_Level)
                server.start()

            ThreadingTCPServer.StartThreadingTCPServer(self, IP, Port)
        except KeyboardInterrupt as e:
            sys.exit(self)

    @classmethod
    def StopSrver(self):
        try:
            self.server.shutdown()  # 停止接受新的连接
            self.server.socket.close()  # 关闭服务器套接字
        except Exception as e:
            pass



















import socket
import socketserver
import subprocess
import threading,multiprocessing


class ThreadingTCPServer(socketserver.BaseRequestHandler):
    """
        启动TCP 服务底层基于socketserver
        Starting TCP service is based on socketserver.
    """
    
    @classmethod
    def StartThreadingTCPServer(self, cls, IP, Port) -> None:
        if type(Port)==int:
            self.server = socketserver.ThreadingTCPServer((IP,Port),cls)   #创建socket链接
            self.server.allow_reuse_address = True
            print(f"Server Stared on {IP}: {Port}")
            self.server.serve_forever()
            
        if type(Port)==list:
            for Pr in Port:
                self.server = socketserver.ThreadingTCPServer((IP,Pr),cls)   #创建socket链接
                self.server.allow_reuse_address = True
                print(f"Server Stared on {IP}: {Pr}")
                # 创建一个线程来启动每个服务器
                thread = threading.Thread(target=self.server.serve_forever)
                # 启动线程
                thread.start()


    def stop_server(self):
        try:
            self.server.shutdown()
            self.server.server_close()
        except Exception as e:
            print(f"Server Close")







import re
import json


class ElementOperation:
    """
        元素操作 
        Element operation
    """

    def is_displayed(self, xpath: str) -> bool:
        """
            元素是否可见
            Is the element visible

            xpath: xpath 路径
            return: 布尔值

            xpath: xpath path
            return: bool
        """
        return "true" in self.SendData("isDisplayed", xpath)

    def is_available(self, xpath: str) -> bool:
        """
            判断元素是否可用
            Determines whether an element is available.

            xpath: xpath 路径
            return: 布尔值

            xpath: xpath path
            return: bool
        """
        return "true" in self.SendData("isEnabled", xpath)

    def is_selected(self, xpath: str) -> bool:
        """
            元素是否已选中
            Whether the element is selected

            xpath: xpath 路径
            return: 布尔值

            xpath: xpath path
            return: bool
        """

        return "true" in self.SendData("isSelected", xpath)

    def get_element_outer_html(self, xpath: str) -> str:
        """
            获取元素的HTML包含对象本身以及所有子节点
            Gets the HTML of the element including the object itself and all child nodes

            xpath: xpath 路径
            return: HTML 带标签格式文本

            xpath: xpath path
            return: HTML tagged text
        """
        response = self.SendData("getElementOuterHTML", xpath)
        if response == "null":
            return None
        return response

    def get_element_inner_html(self, xpath: str) -> str:
        """
            获取元素的HTML所有子节点不包含对象本身
            Gets the HTML of the element. All child nodes do not contain the object itself

            xpath: xpath 路径
            return: HTML 带标签格式文本

            xpath: xpath path
            return: HTML tagged text
        """
        response = self.SendData("getElementInnerHTML", xpath)
        if response == "null":
            return None
        return response

    def get_element_text(self, xpath: str) -> str:
        """
            获取元素文本
            Get element text

            xpath: 元素的 xpath 路径
            return: 元素文本字符串或 None

            xpath: xpath path of the element
            return: element text string or None
        """
        response = self.SendData("getElementText", xpath)
        if "null" in response:
            return None
        return response

    def get_element_attr(self, xpath: str, attr_name: str) -> str:
        """
            获取元素HTML属性
            Gets the HTML attribute of the element

            xpath: xpath 路径
            attr_name: 属性名称
            return: 字符串

            xpath: xpath path
            attr_name: attribute name
            return: string
        """
        response = self.SendData("getElementAttribute", xpath, attr_name)
        if response == "null":
            return None
        return response

    def get_element_value(self, xpath: str) -> str:
        """
            获取input编辑框中的值
            Gets the value in the input edit box

            xpath: input输入框中的xpath路径
            return: 成功返回input输入框的值失败返回None

            xpath: XPath path in the input input box
            return: returns the value of the input input box successfully, and returns None if it fails
        """

        command =   """(function () {\
            let element = document.evaluate('"""
            
        command2 = """', document).iterateNext();\
            if(element == null)\
                return null;\
            else\
                return element.value;\
            })()"""
        command3 = command+xpath+command2
        response = self.execute_script(command3)
        if response == "None":
            return None
        return response

    def click_element(self, xpath: str) -> bool:
        """
            点击元素
            Click element

            xpath: 元素的 xpath 路径
            return: 布尔值

            xpath:  xpath path of the element
            return: Boolean value
        """
        return "true" in self.SendData("clickElement", xpath)

    def clear_element(self, xpath: str) -> bool:
        """
            清除元素值
            Clear element value

            xpath: xpath 路径
            return: 布尔值

            xpath: xpath path
            return: bool
        """
        return "true" in self.SendData("clearElement", xpath)

    def set_element_focus(self, xpath: str) -> bool:
        """
            设置元素焦点
            Set element focus

            xpath: xpath 路径
            return: 布尔值

            xpath: xpath path
            return: bool
        """
        return "true" in self.SendData("setElementFocus", xpath)

    def send_keys(self, xpath: str, value: str) -> bool:
        """
            用于模拟键盘输入，输入的字符会往后叠加；如果元素不能设置焦点，应先 click_mouse 点击元素获得焦点后再输入
            Used to simulate keyboard input, and the input characters will be superimposed later; If the element can't set the focus, click_mouse to get the focus before entering

            xpath: 元素 xpath 路径
            value: 输入的内容
            return: 布尔值

            xpath: xpath path of element
            value: the content entered
            return: Boolean value
        """
        return "true" in self.SendData("sendKeys", xpath, value)

    def set_element_value(self, xpath: str, value: str) -> bool:
        """
            设置元素值 (与send_keys原理一样不同的是set_element_value不会往后叠加字符)
            Set the element value (the same difference as the principle of send_keys is that set_element_value does not superimpose characters later)

            xpath: 元素 xpath 路径
            value: 设置的内容
            return: 布尔值

            xpath: xpath path of element
            value: the content of the setting
            return: Boolean value
        """
        return "true" in self.SendData("setElementValue", xpath, value)

    def send_vk(self, vk: str) -> bool:
        """
            发送Vk虚拟键码, 按键对照表: http://www.atoolbox.net/Tool.php?Id=815
            Send Vk virtual key code Key comparison table

            vk: 输入内容
            return: 布尔值

            vk: input content   http://www.atoolbox.net/Tool.php?Id=815
            return: Boolean value.
        """
        return "true" in self.SendData("sendVk", vk)

    def set_element_attr(self, xpath: str, attr_name: str, attr_value: str) -> bool:
        """
            设置元素属性
            Set element properties

            xpath: 元素 xpath 路径
            attr_name: 属性名称
            attr_value: 属性值
            return: 布尔值

            xpath: xpath path of element
            attr_name: attribute name
            attr_value: attribute value
            return: Boolean value
        """
        return "true" in self.SendData("setElementAttribute", xpath, attr_name, attr_value)

    def upload_file_by_element(self, xpath: str, file_path: str) -> bool:
        """
            通过元素上传文件
            Upload files through elements

            xpath:  元素 xpath 路径
            file_path: 文件路径
            return: 布尔值

            xpath: xpath path of element
            file_path: file path
            return: bool
        """
        return "true" in self.SendData("uploadFile", xpath, file_path)

    def get_element_rect(self, xpath: str) -> tuple:
        """
            获取元素矩形坐标
            Get rectangular coordinates of elements

            xpath: xpath 路径
            return: 元素矩形坐标或None

            xpath: xpath path
            return: Element rectangular coordinates or None
        """
        response = self.SendData("getElementRect", xpath)
        if response == "null":
            return None
        Coordinate = json.loads(response)
        L = Coordinate.get("left")
        R = Coordinate.get("right")

        T = Coordinate.get("top")
        B = Coordinate.get("bottom")
        response = (L+(R-L)/2, T+(B-T)/2)
        return response

    def save_screenshot(self, xpath: str = None, path: str = 'PyAibote.png') -> str:
        """
            截图，返回 PNG 格式的 base64, 保存图片时尽量不要把浏览器开全屏会出现异常抛错情况
            Screenshot, return to base64 in PNG format

            xpath: 元素路径，如果指定该参数则截取元素图片
            path: 生成图片路径，如果不设置路径则默认在当前目录生成名为PyAibote.png图片,如果设置为空值则不生成图片
            return: PNG 格式的 base64 的字符串或 None

            xpath: the path of the element. If this parameter is specified, the image of the element will be clipped
            path: Generate the image path. If not specified, the image named PyAibote.png will be generated in the current directory by default
            return: string of base64 in PNG format or None
        """
        if xpath is None:
            response = self.SendData("takeScreenshot")
        else:
            response = self.SendData("takeScreenshot", xpath)
        if response == "null":
            return None
        if path:
            self.SaveBase64Png(response,path)
        return response

    def show_xpath(self) -> bool:
        """
            显示元素xpath路径，页面加载完毕再调用。
            调用此函数后，可在页面移动鼠标会显示元素区域。移动并按下ctrl键，会在浏览器控制台打印相对xpath 和 绝对xpath路径
            ifrmae 内的元素，需要先调用 switchFrame 切入进去，再调用showXpath函数

            return: 总是True

            Displays the xpath path of the element, and then calls it after the page is loaded.
            After calling this function, you can move the mouse on the page to display the element area. 
            Moving and pressing the ctrl key will print the relative xpath and absolute xpath paths on the browser console.
            Elements in ifrmae need to be cut in by calling switchFrame first, and then calling the showXpath function.

            return: always True
        """
        return "true" in self.SendData("showXpath")

    def show_xpath(self) -> bool:
        """
            显示元素xpath路径，页面加载完毕再调用。
            调用此函数后，可在页面移动鼠标会显示元素区域。移动并按下ctrl键，会在浏览器控制台打印相对xpath 和 绝对xpath路径
            ifrmae 内的元素，需要先调用 switchFrame 切入进去，再调用showXpath函数

            return: 总是True

            Displays the xpath path of the element, and then calls it after the page is loaded.
            After calling this function, you can move the mouse on the page to display the element area. 
            Moving and pressing the ctrl key will print the relative xpath and absolute xpath paths on the browser console.
            Elements in ifrmae need to be cut in by calling switchFrame first, and then calling the showXpath function.

            return: always True
        """
        return "true" in self.SendData("showXpath")

    def get_elements(self) -> json:
        """
            获取可见区域内的所有元素信息
            return: 成功返回数组json格式的元素信息，失败返回null

            Get information of all elements in the visible area
            return: element information in json format of array is returned successfully, and null is returned if it fails
        """
        response = self.SendData("getElements")
        if response == "null":
            return None
        return json.loads(response)




"""
    写DEBUG调试LOG
    Write DEBUG debug LOG
"""
import os
import time
from loguru import logger
from datetime import datetime


class LoggerRecord:
    """
        根据当天日期储存LOG文件
        Generate LOG file according to today's date.
    """
    
    Log_Storage = False
    Log_Level = "DEBUG"
    Log_Format = f"%(asctime)s - %(levelname)s : %(message)s"
    LOGMainPath = "AiBotRunLOG"
    LOGDayPath = f"AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}"
    DEBUGLogPath = f"{LOGDayPath}/DEBUG.log"
    ERRORLogPath = f"{LOGDayPath}/ERROR.log"
    INFOLogPath = f"{LOGDayPath}/INFO.log"

    def CheckFile(self):
        if self.Log_Storage:
            if not os.path.exists(self.LOGMainPath):
                os.makedirs(self.LOGMainPath)

            if not os.path.exists(self.LOGDayPath):
                os.makedirs(self.LOGDayPath)
            
    def Write_logger(self,LogFilePath,Level,Info) -> None:  # 写入日志信息
        """
            将LOG信息写入文件
            Write LOG information to a file.

            LogFilePath: LOG文件路径
            Info: LOG的信息
            return: 空

            LogFilePath: LOG file path
            Info: LOG information
            return: None
        """
        current_time = datetime.now()
        formatted_time = current_time.strftime("%Y-%m-%d %H:%M:%S.%f")
        with open(LogFilePath,"a", encoding="UTF-8") as w:
            w.write(f"{formatted_time} - {Level} : {Info}\n")

    def debug(self, Msg) -> None:
        if self.Log_Storage:
            self.CheckFile()
            self.Write_logger(self.DEBUGLogPath, "DEBUG" ,Msg)

        if self.Log_Level == "DEBUG":
            logger.debug(Msg)

    def info(self, Msg) -> None:
        if self.Log_Storage:
            self.CheckFile()
            self.Write_logger(self.INFOLogPath, "INFO", Msg)

        if self.Log_Level == "INFO":
            logger.info(Msg)

    def error(self, Msg) -> None:
        if self.Log_Storage:
            self.CheckFile()
            self.Write_logger(self.ERRORLogPath, "ERROR", Msg)
        logger.error(Msg)

        


import re


class NewDigitalHumanOperation:
    """
        新版数字人
        New Digital human
    """

    def init_new_metahuman(self, model_folder: str, scale: int, is_paly_media_audio: bool, enable_random_param: bool, push_stream_url: bool) -> bool:
        """
            初始化数字人
            Initialize digital person

            model_folder: 模型文件夹
            scale: 缩放，1原始大小，0.5缩小一半
            is_paly_media_audio: 是否播放素材中的音频
            enable_random_param: 是否开启随机参数，随机内容：音频随机(不含数字人说话声音随机) 和 画面随机(包含平移、旋转、缩放、动作泛化)
            push_stream_url: 推流地址，默认为 "" 不推流，仅支持rtmp，例如： rtmp://your_server_ip/live/stream
            return: 成功返回True，失败返回错误信息

            model_folder: model folder
            scale: scaling, 1 original size, 0.5 reduced by half
            is_paly_media_audio: whether to play the audio in the material
            Enable_random_param: whether to turn on the random parameter, including audio random (excluding digital human voice random) 
                                 and picture random (including translation, rotation, scaling and motion generalization).
            push_stream_url: push stream address; the default value is "",and only rtmp is supported; for example, RTMP://your _ server _ IP/live/stream
            return: Returns True on success, and returns an error message on failure
        """
        return "true" in self.SendData("initNewHuman", model_folder, scale, is_paly_media_audio, enable_random_param, push_stream_url) 

    def new_get_face_data(self, server_ip: str, call_api_key: str, video_path: str) -> bool:
        """
            获取脸部数据
            Get face data

            server_ip: 服务端IP
            call_api_key: 调用密钥
            video_path: 人脸视频路径
            return:  失败返回错误信息，成功返回true。并在videoPath同级目录下生成.pt 后缀的人脸数据文件

            server_ip: server IP
            call_api_key: call key
            video_path: face video path
            return: failure returns an error message, and success returns true. And generate a face data file with. pt suffix in the same level directory of videoPath
        """
        return "true" in self.SendData("getFaceData", server_ip, call_api_key, video_path) 

    def new_metahuman_switch_action(self, figure_video_path: str, scale: int, is_paly_media_audio: bool, is_swap_color: bool, wait_play_end: bool) -> bool:
        """
            切换人物形象动作 (使用前需要先调用 init_new_metahuman 初始化数字人)
            Switch character image action

            figure_video_path: 人物视频路径
            scale: 缩放，1原始大小，0.5缩小一半
            is_paly_media_audio: 是否播放素材中的音频
            is_swap_color: 是否更换基础嘴型颜色
            wait_play_end: 等待素材播放完毕再切换，保证动作无缝衔接
            return: 成功返回True，失败返回错误信息

            figure_video_path: model folder
            scale: scaling, 1 original size, 0.5 reduced by half
            is_paly_media_audio: whether to play the audio in the material
            is_swap_color: Do you want to change the basic mouth color
            wait_play_end: Wait for the material to be played before switching to ensure the seamless connection of actions
            return: Returns True on success, and returns an error message on failure
        """
        return "true" in self.SendData("switchAction", figure_video_path, scale, is_paly_media_audio, is_swap_color, wait_play_end) 

    def new_metahuman_add_background(self, bg_path: str, x: int, y: int) -> bool:
        """
            添加视频/图片背景 (使用前需要先调用 init_new_metahuman 初始化数字人)
            Add video/picture background

            bg_path: 背景视频/图片路径
            x: 人物起始横坐标, 默认为0
            y: 人物起始纵坐标, 默认为0
            return: 成功返回True，失败返回错误信息

            bg_path: background video/picture path
            x: horizontal coordinate of character start, which is 0 by default
            y: the starting ordinate of the character, which is 0 by default
            return: Returns True on success, and returns an error message on failure
        """
        response = self.SendData("addBackground", bg_path, x, y) 
        if "true" in response:
            return True
        return response

    def new_metahuman_del_background(self) -> bool:
        """
            删除视频/图片背景
            Delete video/picture background

            return: 成功返回True，失败返回错误信息
            return: Returns True on success, and returns an error message on failure
        """
        return "true" in self.SendData("delBackground") 

    def new_metahuman_generate_human_video(self, audio_path: str) -> bool:
        """
            生成数字人视频
            Generate digital human video

            audio_path: 音频路径，需要提前生成lab文件
            return: 失败返回错误信息，成功返回true，并在audio_path同级目录下生成同名 .mp4 后缀的数字人视频文件

            audio_path: audio path. lab files need to be generated in advance.
            return: an error message is returned in case of failure, and true is returned in case of success, and a digital human video file with the same name and .mp4 suffix is generated in the same level directory of audio_path

        """
        return "true" in self.SendData("generateHumanVideo", audio_path) 

    def new_metahuman_generate_human_video_ex(self, server_ip: str, call_api_key: str, audio_path: str, video_path: str, save_video_path: str, is_music: bool = False) -> bool:
        """
            云端算力生成短视频
            Cloud computing generates short videos

            server_ip: 服务端IP
            call_api_key: 调用密钥
            audio_path: 驱动口型的音频路径
            video_path: 原视频路径
            save_video_path: 保存的合成结果路径
            is_music: 生成唱歌视频，要求 audio_path 音乐音频
            return: 失败返回错误信息，成功返回True

            server_ip: server IP
            call_api_key: call key
            audio_path: the audio path of the driving mouth
            video_path: original video path
            save_video_path: the saved composition result path
            is_music: to generate singing video, audio_path music audio is required
            return: failure returns an error message, and success returns true
        """
        return "true" in self.SendData("generateHumanVideoEx", server_ip, call_api_key, audio_path, video_path, save_video_path, is_music) 

    def new_metahuman_audio_to_lab(self, server_ip: str, audio_path: str) -> bool:
        """
            生成lab文件
            Generate lab file

            server_ip: lab服务端IP
            audio_path: 音频文件路径
            return: 失败返回错误信息，成功返回true，并在audioPath同级目录下生成同名 .lab 后缀的lab文件

            server_ip: lab server IP
            audio_path: audio file path
            return: failure returns an error message, success returns true, and a lab file with the same name. lab suffix is generated in the same directory of audioPath
        """
        return "true" in self.SendData("audioToLab", server_ip, audio_path) 

    def new_metahuman_text_to_audio(self, server_ip: str, save_audio_path: str, refer_audio_path: str, refer_text: str, text : str, speed_factor : int) -> bool:
        """
            文本转音频
            Text to audio

            server_ip: 声音克隆服务端IP
            save_audio_path: 保存的音频文件路径
            refer_audio_path: 参考音频路径
            refer_text: 参考文本内容(内容必须与参考音频播报的内容一致)
            text: 要合成的文本内容
            speed_factor: 语速，默认为1正常语速
            return: 失败返回错误信息，成功返回true

            server_ip: voice cloning server IP
            save_audio_path: the path of the saved audio file
            reference _ audio _ path: the refer_audio_path
            refer_text: Reference text content (the content must be consistent with the content broadcast by reference audio)
            text: the text content to be synthesized
            speed_factor: speech speed, which defaults to 1 normal speech speed
            return: failure returns an error message, and success returns true
        """
        return "true" in self.SendData("textToAudio", server_ip, save_audio_path, refer_audio_path, refer_text, text, speed_factor) 
    
    def new_metahuman_audio_to_text(self, server_ip: str, audio_path: str) -> bool:
        """
            语音转文本
            Speech to text

            server_ip: 语音识别服务端IP
            audio_path: 音频文件路径
            return: 失败返回"None"，成功返回语音识别的文本内容

            server_ip: IP of speech recognition server
            audio_path: audio file path
            return: "None" is returned in case of failure, and the text content of speech recognition is returned successfully
        """
        response = self.SendData("audioToText", server_ip, audio_path) 
        if response == "null":
            return None
        return response
    
    def new_metahuman_human_speak(self, audio_path: str, wait_play_sound: bool, insert_head: bool = False) -> bool:
        """
            数字人说话
            Digital people speak

            audio_path: 音频路径，需要提前生成lab文件
            wait_play_sound: 是否等待播报完毕
            insert_head: 插入队列头部,优先播报
            return: 成功返回true，失败返回错误信息

            audio_path: audio path. lab files need to be generated in advance
            wait_play_sound: Do you want to wait for the broadcast to finish
            insert_head: Insert the head of the queue and broadcast first
            return: Returns true on success, and returns an error message on failure
        """
        response = self.SendData("humanSpeak", audio_path, wait_play_sound, insert_head) 
        if "true" in response:
            return True
        return response
    
    def new_metahuman_stop_speak(self) -> bool:
        """
            打断说话
            Interrupt speech

            return: 返回true
            return: Returns true
        """
        return "true" in self.SendData("stopSpeak") 
    
    def new_metahuman_start_record(self, save_audio_path: str) -> bool:
        """
            录制麦克风
            Recording microphone

            save_audio_path: 录制保存的音频路径，需要调用 stopRecord 结束录制并保存
            return: 返回true

            save_audio_path: record the saved audio path. You need to call stopRecord to end the recording and save it
            return: returns true
        """
        return "true" in self.SendData("startRecord", save_audio_path) 
    
    def new_metahuman_stop_record(self) -> bool:
        """
            停止录制
            Stop recording

            return: 返回true
            return: returns true
        """
        return "true" in self.SendData("stopRecord") 

    def new_metahuman_train_voice_ex(self, appid: str, token: str, spk_id: str, refer_audio_path: str) -> bool:
        """
            云端算力训练声音
            Cloud computing power training voice

            appid: APP ID
            token: Access Token
            spk_id: 声音ID
            refer_audio_path: 参考音频
            return: 训练成功返回true，失败返回错误信息

            appid: APP ID
            token: Access Token
            spk_ID: sound id
            reference_audio_path: reference audio
            return: training success returns true, failure returns an error message
        """
        response = self.SendData("trainVoiceEx", appid, token, spk_id, refer_audio_path) 
        if "true" in response:
            return True
        return response
    
    def new_metahuman_get_train_status_ex(self, appid: str, token: str, spk_id: str) -> str:
        """
            获取 trainVoiceEx 训练状态
            Get the trainVoiceEx training status

            appid: APP ID
            token: Access Token
            spk_id: 声音ID
            return: 返回训练状态"Train Success"、"NotFound"、"Training"、"Failed"、unknow"   

            appid: APP ID
            token: Access Token
            spk_ID: sound id
            return: returns to the Training status of "Train Success", "NotFound", "training", "Failed" and "unknow"
        """
        response = self.SendData("getTrainStatusEx", appid, token, spk_id) 
        return response
    
    def new_metahuman_text_to_audio_ex(self, appid: str, token: str, spk_id: str, cluster: str, text: str, speed_ratio: str, save_audio_path: str, language: str = 'zh-cn') -> bool:
        """
            云端算力文本转语音
            Cloud computing power text-to-speech

            appid: APP ID
            token: Access Token
            spk_id: 声音ID
            cluster: Cluster ID  声音复刻大模型："volcano_icl"  语音合成大模型："volcano_tts"
            text: 合成的文本
            speed_ratio: 语速，正常为1
            save_audio_path: 保存的音频路径， 同时会在音频同目录下生成lab文件
            return: 成功返回true，失败返回错误信息

            appid: APP ID
            token: Access Token
            spk_ID: sound id
            cluster: Cluster ID sound reproduction model: "volcano_icl" speech synthesis model: "volcano_tts"
            text: synthesized text
            speed_ratio: speech speed, which is normally 1
            save_audio_path: the saved audio path, and a lab file will be generated in the same audio directory
            return: Returns true on success, and returns an error message on failure
        """
        response = self.SendData("textToAudioEx", appid, token, spk_id, cluster, text, speed_ratio, save_audio_path, language) 
        if "true" in response:
            return True
        return response

    def new_metahuman_get_extend_param(self) -> str:
        """
            获取驱动程序命令行参数(不包含ip和port)
            Get the driver command line parameters (excluding ip and port)

            return: 成功返回参数，失败返回None
            return: parameter is returned successfully, and None is returned on failure.
        """
        response = self.SendData("getExtendParam")
        if response == "null":
            return None
        return response
    
    def new_metahuman_get_driver_folder(self) -> str:
        """
            获取驱动程序 所在的文件夹路径
            Gets the folder path where the driver is located

            return: 返回AiDriver.exe所在的文件夹路径 
            return: Returns the folder path where AiDriver.exe is located
        """
        response = self.SendData("getDriverFolder") 
        return response

    def new_metahuman_close_driver(self) -> str:
        """
            关闭驱动
            Turn off drive

            return: 返回true
            return: Return true
        """
        response = self.SendData("closeDriver") 
        return response

    def new_metahuman_insert_video(self, video_path: str) -> bool:
        """
            插入特写视频
            Insert close-up video

            video_path: 要播放的视频
            return: 成功返回true，失败返回错误信息

            video_path: the video to play
            return: Returns true on success, and returns an error message on failure
        """
        response = self.SendData("insertVideo", video_path) 
        if "true" in response:
            return True
        return response
    
    def new_metahuman_get_human_speak_queue_count(self) -> str:
        """
            获取数字人说话 队列存入的数量
            Get the number of digital people talking queues

            return: 返回队列存入的数量
            return: Returns the amount deposited in the queue
        """
        response = self.SendData("getHumanSpeakQueueCount") 
        return response
    
    def new_metahuman_train_base_model(self, server_ip: str, call_api_key: str, video_or_image_path: str, save_folder: str) -> bool:
        """
            训练基础模型
            Training basic model

            server_ip: 服务端IP
            call_api_key: 调用密钥
            video_or_image_path: 人脸 图片/视频 路径，图片/视频 第一帧必须包含人脸且正脸对着摄像头
            save_folder: 模型保存的文件夹路径
            return: 失败返回错误信息，成功返回True。模型保存在 saveFolder 文件夹下。一般耗时60分钟左右

            server_ip: server IP
            call_api_key: call key
            video_or_image_path: the path of face picture/video. The first frame of picture/video must contain face and face the camera
            save_folder: the folder path where the model is saved
            return: failure returns an error message, and success returns true
                    he model is saved in the saveFolder folder. It usually takes about 60 minutes
        """
        response = self.SendData("trainBaseModel", server_ip, call_api_key, video_or_image_path, save_folder) 
        if "true" in response:
            return True
        return response
    
    def new_metahuman_audio_to_lab_ex(self, access_key_id: str, access_key_secret: str, app_key: str, audio_path: str) -> str:
        """
            云端生成lab文件(对接阿里云语音识别极速版)
            Cloud Generation lab File (Docking Alibaba Cloud Speech Recognition Extreme Edition)

            access_key_id: 密钥ID
            access_key_secret: 密钥
            app_key: app密钥
            audio_path: 音频文件路径
            return: 失败返回错误信息，成功返回True，并在audioPath同级目录下生成同名 .lab 后缀的lab文件  

            access_key_ID: key id
            access_key_secret: key
            app_key: app key
            audio_path: audio file path
            return: failure returns an error message, success returns true, and a lab file with the same name. 
                    lab suffix is generated in the same directory of audioPath
        """
        response = self.SendData("audioToLabEx", access_key_id, access_key_secret, app_key, audio_path) 
        if "true" in response:
            return True
        return response
    
    def new_metahuman_hide_human_window(self, is_hide) -> bool:
        """
            隐藏数字人窗口
            Hidden numbers ren Chuang kou

            is_hide: 是否隐藏, True 隐藏, False 显示
            return: 返回True

            is_hide: whether to hide, True to hide, false to display
            return: Returns True
        """
        return "true" in self.SendData("hideHumanWindow", is_hide) 
    
    def new_metahuman_del_human_window_border(self, is_no_border) -> bool:
        """
            删除数字人窗口标题栏边框
            Delete the title bar border of the Digital Man window

            is_no_border: 是否删除标题栏边框, True 删除标题栏, False 恢复
            return: 返回True

            is_no_border: whether to delete the title bar border, true to delete the title bar, false to restore
            return: returns true
        """
        return "true" in self.SendData("delHumanWindowBorder", is_no_border) 
    
    def new_metahuman_llm(self, app_id: str, api_key: str, prompt: str) -> str:
        """
            通义千问(应用)
            General meaning and thousand questions (application)

            app_id: 应用ID
            api_key: API密钥
            prompt: 提问
            return: 成功输出语言模型答案，失败返回错误信息 

            app_id: application ID
            api_key: API key
            prompt: ask questions
            return: the language model answer is successfully output, 
                     and an error message is returned if it fails
        """
        response = self.SendData("llm", app_id, api_key, prompt) 
        return response
    
    def new_metahuman_get_face_data_service(self, video_path: str) -> str:
        """
            获取脸部数据(中转服务)
            Get face data (transit service)

            video_path: 人脸视频路径
            return: 失败返回错误信息，成功返回True。并在videoPath同级目录下生成.pt 后缀的人脸数据文件 

            video_path: face video path.
            return: failure returns an error message, and success returns true. 
                    And generate a face data file with. pt suffix in the same level directory of videoPath.
        """
        response = self.SendData("getFaceData_Service", video_path) 
        if "true" in response:
            return True
        return response

    def new_metahuman_train_base_model_service(self, video_or_image_path: str, save_folder: str) -> str:
        """
            训练基础模型(中转服务)
            Training basic model (transit service)

            video_or_image_path: 人脸 图片/视频 路径，图片/视频 第一帧必须包含人脸且正脸对着摄像头
            save_folder: 模型保存的文件夹路径
            return: 失败返回错误信息，成功返回True。模型保存在 saveFolder 文件夹下。一般耗时60分钟左右

            video_or_image_path: the path of face picture/video. The first frame of picture/video must contain face and face the camera.
            save_folder: the folder path where the model is saved.
            return: failure returns an error message, and success returns true. 
                    The model is saved in the saveFolder folder. It usually takes about 60 minutes.
        """
        response = self.SendData("trainBaseModel_Service", video_or_image_path, save_folder) 
        if "true" in response:
            return True
        return response
    
    def new_metahuman_generate_human_video_ex_service(self, audio_path: str, video_path: str, save_video_path: str, is_music: bool = False) -> str:
        """
            云端算力生成短视频(中转服务)
            Cloud computing generates short video (transit service)

            audio_path: 驱动口型的音频路径
            video_path: 原视频路径
            save_video_path: 保存的合成结果路径
            is_music: 生成唱歌视频，要求 audio_path 音乐音频
            return: 失败返回错误信息，成功返回True

            audio_path: the audio path of the driving mouth.
            video_path: original video path.
            save_video_path: the saved composition result path.
            is_music: to generate singing video, audio_path music audio is required.
            return: failure returns an error message, and success returns true.
        """
        response = self.SendData("generateHumanVideoEx_Service", audio_path, video_path, save_video_path, is_music) 
        if "true" in response:
            return True
        return response
    
    def new_metahuman_train_voice_ex_service(self, refer_audio_path: str) -> str:
        """
            云端算力训练声音(火山引擎)(中转服务)
            Cloud Computing Training Voice (Volcano Engine) (Transit Service)

            refer_audio_path: 参考音频
            return: 训练成功返回True，失败返回错误信息

            refer_audio_path: reference audio.
            return: training success returns true, failure returns an error message.
        """
        response = self.SendData("trainVoiceEx_Service", refer_audio_path) 
        if "true" in response:
            return True
        return response

    def new_metahuman_text_to_audio_ex_service(self, spk_id: str, cluster: str, text: str, speed_ratio: str, save_audio_path: str, language: str = 'zh-cn') -> str:
        """
            云端算力文本转语音(火山引擎)(中转服务)
            Cloud Computing Text-to-Speech (Volcano Engine) (Transit Service)

            spk_id: 声音复刻大模型的声音ID 语音合成大模型的Voice_type
            cluster: Cluster ID  声音复刻大模型："volcano_icl"     语音合成大模型："volcano_tts"
            text: 合成的文本
            speed_ratio: 语速，正常为1
            save_audio_path: 保存的音频路径， 同时会在音频同目录下生成lab文件
            language: 语言，中文（zh-cn）、英语（en）、西班牙语（es）、法语（fr）、德语（de）、俄语（ru）、日语（ja）、阿拉伯语（ar）、印地语（hi）、葡萄牙语（pt）
            return: 失败返回错误信息，成功返回True

            spk_ID: the voice id of the voice reproduction model and the Voice_type of the speech synthesis model.
            cluster: Cluster ID sound reproduction model: "volcano_icl" speech synthesis model: "volcano_tts"
            text: synthesized text
            speed_ratio: speech speed, which is normally 1.
            save_audio_path: the saved audio path, and a lab file will be generated in the same audio directory.
            language: languages: Chinese (zh-cn), English (en), Spanish (es), 
                     French (fr), German (de), Russian (ru), Japanese (ja), Arabic (ar), hindi (Hi) and Portuguese (pt).
            return: failure returns an error message, and success returns true.
        """
        response = self.SendData("textToAudioEx_Service", spk_id, cluster, text, speed_ratio, save_audio_path, language) 
        if "true" in response:
            return True
        return response

    def new_metahuman_audio_to_lab_ex_service(self, audio_path: str) -> str:
        """
            云端生成lab文件(对接阿里云语音识别极速版)(中转服务)
            Cloud-generated lab file (connected to Alibaba Cloud Speech Recognition Extreme Edition) (transit service)

            audio_path: 音频文件路径
            return: 失败返回错误信息，成功返回True，并在audioPath同级目录下生成同名 .lab 后缀的lab文件

            audio_path: audio file path.
            return: failure returns an error message, success returns true, and a lab file with the same name. 
                    lab suffix is generated in the same directory of audioPath.
        """
        response = self.SendData("audioToLabEx_Service", audio_path) 
        if "true" in response:
            return True
        return response

    def new_metahuman_llm_service(self, prompt: str) -> str:
        """
            通义千问(应用)(中转服务)
            Tong yi Qian Wen (application) (transit service)

            prompt: 提问
            return: 成功输出语言模型答案，失败返回错误信息

            prompt: ask questions
            return: successfully output the language model answer, and return an error message if it fails.
        """
        response = self.SendData("llm_Service", prompt) 
        return response
    
    def new_metahuman_get_windows_id(self) -> str:
        """
            获取 windows ID
            get windows ID

            return: 成功返回windows Id
            return: successfully returned windows Id
        """
        response = self.SendData("getWindowsId") 
        return response
    
    def new_metahuman_blur_mouth(self, is_blur_mouth: str) -> str:
        """
            高斯模糊嘴型，优化嘴型细节瑕疵
            Gaussian fuzzy mouth shape, optimizing mouth shape detail defects

            isBlurMouth: 开启后淡化口型瑕疵细节，嘴型清晰度会降低
            return: 成功返回true

            isBlurMouth: After opening, the details of the mouth defects will be diluted, 
                        and the mouth definition will be reduced
            return: returned true successfully
        """
        return "true" in self.SendData("blurMouth", is_blur_mouth) 


import re
import json

class Control:
    """
        控件操作
        Control operation
    """
    def create_text_view(self, _id: int, text: str, coordinate: tuple = (400,500), width: int = 400, height: int = 60) -> bool:
        """
            创建文本框控件
            Create a text box control

            _id:  控件ID，不可与其他控件重复
            text:  控件文本
            coordinate   默认 (400,500)
            width:  控件宽度，默认 400
            height:  控件高度，默认 60
            return: True或者False

            _id: control id, which cannot be duplicated with other controls
            text: control text
            coordinate default (400,500)
            width: the width of the control, which is 400 by default
            height: control height, 60 by default
            return: True or False
        """
        return "true" in self.SendData("createTextView", _id, text, coordinate[0], coordinate[1], width, height)

    def create_edit_view(self, _id: int, text: str, coordinate: tuple = (400,500), width: int = 400, height: int = 150) -> bool:
        """
            创建编辑框控件
            Create an edit box control

            _id:  控件ID，不可与其他控件重复
            text:  控件文本
            coordinate: x,y坐标 默认 (400,500)
            width:  控件宽度，默认 400
            height:  控件高度，默认 150
            return: True或者False

            _id: control id, which cannot be duplicated with other controls
            text: control text
            coordinate: x,y coordinates default (400,500)
            width: the width of the control, which is 400 by default
            height: control height, default is 150
            return: True or False
        """
        return "true" in self.SendData("createEditText", _id, text, coordinate[0], coordinate[1], width, height) 

    def create_check_box(self, _id: int, text: str, coordinate: tuple = (400,500), width: int = 400, height: int = 60,is_select: bool = False) -> bool:
        """
            创建复选框控件
            Create a check box control

            _id:  控件ID，不可与其他控件重复
            text:  控件文本
            coordinate:  x,y坐标 默认 (400,500)
            width:  控件宽度，默认 400
            height:  控件高度，默认 60
            is_select:  是否勾选，默认 False
            return: True或者False

            _id: control id, which cannot be duplicated with other controls
            text: control text
            coordinate: x,y coordinates default (400,500)
            width: the width of the control, which is 400 by default
            height: control height, default is 60
            is_select: whether it is checked or not; the default value is False
            return: True or False
        """
        return "true" in self.SendData("createCheckBox", _id, text, coordinate[0], coordinate[1], width, height, is_select) 

    def create_switch_button(self, _id: int, text: str, coordinate: tuple = (400,500), width: int = 400, height: int = 60,is_select: bool = False) -> bool:
        """
            创建SwitchButton控件
            Create a SwitchButton control

            _id:  控件ID，不可与其他控件重复
            text:  控件文本
            coordinate:  x,y坐标 默认 (400,500)
            width:  控件宽度，默认 400
            height:  控件高度，默认 60
            is_select:  是否开/关 默认 关
            return: True或者False

            _id: control id, which cannot be duplicated with other controls
            text: control text
            coordinate: x,y coordinates default (400,500)
            width: the width of the control, which is 400 by default
            height: control height, default is 60
            is_select: Whether to turn on/off the default off？
            return: True or False
        """
        return "true" in self.SendData("createSwitchButton", _id, text, coordinate[0], coordinate[1], width, height, is_select) 

    def create_list_text(self, _id: int, hint_text: str, list_text: str, coordinate: tuple = (400,500), width: int = 400, height: int = 400) -> bool:
        """
            创建ListText控件
            Create ListText control

            _id:  控件ID，不可与其他控件重复
            hint_text:  提示文本
            list_text:  下拉列表中的文本以逗号分割
            coordinate: x,y坐标 默认 (400,500)
            width:  控件宽度
            height:  控件高度
            return: True或者False

            _ID: control id, which cannot be duplicated with other controls
            Hint_text: prompt text
            List_text: list text
            Coordinate: x,y coordinates default (400,500)
            Width: control width
            Height: control height
            Return: True or False
        """
        return "true" in self.SendData("createListText", _id, hint_text, coordinate[0], coordinate[1], width, height, list_text) 

    def create_web_view(self, _id: int, url: str, coordinate: tuple = (-1,-1), controlsize: tuple = (-1,-1)) -> bool:
        """
            创建WebView控件(直接会打开浏览器进入页面)
            Create a WebView control (which will directly open the browser to enter the page)

            _id: 控件ID，不可与其他控件重复
            url: 加载的链接
            coordinate: 控件在屏幕上 x 坐标，值为 -1 时自动填充宽高
            controlsize: 控件宽,高度，值为 -1 时自动填充宽高
            return: True或者False

            _id: control id, which cannot be duplicated with other controls
            url: loaded link
            coordinate: the x coordinate of the control on the screen, and the width and height are automatically filled when the value is -1
            controlsize: the width and height of the control. When the value is -1, the width and height will be automatically filled
            return: True or False
        """
        return "true" in self.SendData("createWebView", _id, url, coordinate[0], coordinate[1], controlsize[0], controlsize[1]) 

    def get_script_params(self) -> dict:
        """
            获取脚本配置参数(等待用户提交控件数据)
            Get script configuration parameters (wait for the user to submit control data)

            return: 用户提交的控件数据
            return: control data submitted by the user.
        """
        response = self.SendData("getScriptParam")
        if response == "null":
            return None
        try:
            params = json.loads(response)
        except Exception as e:
            return {}
        return params

    def clear_script_widget(self) -> bool:
        """
            清除脚本控件
            Clear script control

            return: True或者False
            return: True or False
        """
        return "true" in self.SendData("clearScriptControl") 


























import base64
from PIL import Image
from io import BytesIO
import psutil

class UniversalFunction:

    def SaveBase64Png(self, Base64Data, OutputPath):
        try:
            imgData = base64.b64decode(Base64Data)
            with open(OutputPath, "wb") as f:
                f.write(imgData)
        except Exception as e:
            print(f"Failed to report the wrong picture. Please save it yourself ErrorMsg: {e}")

    def check_process(self, process_name):
        for proc in psutil.process_iter(['pid', 'name']):
            if process_name == proc.info['name']:
                return True
        return False
    


    





import re


class DigitalHumanOperation:
    """
        数字人
        Digital human
    """

    def init_metahuman(self, metahuman_mde_path: str, metahuman_scale_width: int, metahuman_scale_height: int, is_update_metahuman: bool = False, enable_random_image: bool = False ) -> bool:
        """
            初始化数字人，第一次初始化需要一些时间
            Initializing digital people, it takes some time to initialize for the first time.

            metahuman_mde_path: 数字人模型路径
            metahuman_scale_width: 数字人宽度缩放倍数，1为原始大小。为2时放大一倍，0.5则缩小一半
            metahuman_scale_height: 数字人高度缩放倍数，1为原始大小。为2时放大一倍，0.5则缩小一半
            is_update_metahuman: 是否强制更新，默认fasle。为true时强制更新会拖慢初始化速度
            enable_random_image: 是否启用随机对比度、亮度和形变 参数，默认Fasle
            return: True或者False

            metahuman_mde_path: the path of digital human model
            metahuman_scale_width: the scaling factor of the digital person width, where 1 is the original size. When it is 2, it is doubled, and when it is 0.5, it is halved
            metahuman_scale_height: the scaling factor of the digital person height, where 1 is the original size. When it is 2, it is doubled, and when it is 0.5, it is halved
            is_update_metahuman: whether to force the update, the default is fasle. When true, forcing the update will slow down the initialization speed
            enable_random_image: whether to enable random contrast, brightness and deformation parameters, the default is Fasle
            return: True or False
        """
        return "true" in self.SendData("initMetahuman", metahuman_mde_path, metahuman_scale_width, metahuman_scale_height, is_update_metahuman, enable_random_image) 

    def metahuman_speech(self, save_voice_folder: str, text: str, language: str, voice_name: str, quality: int = 0, wait_play_sound: bool = True, speech_rate: int = 0, voice_style: str = "General") -> bool:
        """
            数字人说话，此函数需要调用 initSpeechService 初始化语音服务
            Digital people speak, this function needs to call initSpeechService to initialize voice service

            save_voice_folder: 保存的发音文件目录，文件名以0开始依次增加，扩展为.mp3格式
            text: 要转换语音的文本
            language: 语言，参考开发文档 语言和发音人
            voice_name: 发音人，参考开发文档 语言和发音人
            quality: 音质，0低品质  1中品质  2高品质， 默认为0低品质
            wait_play_sound: 等待音频播报完毕，默认为 true等待
            speech_rate:  语速，默认为0，取值范围 -100 至 200
            voice_style: 语音风格，默认General常规风格，其他风格参考开发文档 语言和发音人
            return: True或者False

            save_voice_folder: the directory of saved pronunciation files. The file names are increased from 0 and expanded to. wav format
            text: the text to be converted into speech
            language: language, reference development document language and speaker
            Voice_name: speaker, refer to the development document language and speaker
            quality: sound quality, 0 low quality 1 medium quality 2 high quality, and 0 low quality by default
            wait_play_sound: Wait until the audio broadcast is finished; the default value is true
            speech_rate: speech speed, which is 0 by default, and the value range is -100 to 200
            voice_style: voice style, the default General style, and other styles refer to the development document language and speaker
            return: True or False
        """
        return  "false" not in self.SendData("metahumanSpeech", save_voice_folder, text, language, voice_name, quality,wait_play_sound, speech_rate, voice_style) 

    def metahuman_speech_cache(self, save_voice_folder: str, text: str, language: str, voice_name: str, quality: int = 0, wait_play_sound: bool = True, speech_rate: int = 0, voice_style: str = "General") -> bool:
        """
            *数字人说话缓存模式，需要调用 initSpeechService 初始化语音服务。函数一般用于常用的话术播报，非常用话术切勿使用，否则内存泄漏
            * Digital people speak in cache mode, which requires calling initSpeechService to initialize voice service. Function is generally used for common speech broadcasting, and it should not be used if it is very popular, otherwise the memory will leak
            
            save_voice_folder: 保存的发音文件目录，文件名以0开始依次增加，扩展为.wav格式
            text: 要转换语音的文本
            language: 语言，参考开发文档 语言和发音人
            voice_name: 发音人，参考开发文档 语言和发音人
            quality: 音质，0低品质  1中品质  2高品质， 默认为0低品质
            wait_play_sound: 等待音频播报完毕，默认为 true等待
            speech_rate:  语速，默认为0，取值范围 -100 至 200
            voice_style: 语音风格，默认General常规风格，其他风格参考开发文档 语言和发音人
            return: True或者False

            save_voice_folder: the directory of saved pronunciation files. The file names are increased from 0 and expanded to. wav format
            text: the text to be converted into speech
            language: language, reference development document language and speaker
            voice_name: speaker, refer to the development document language and speaker
            quality: sound quality, 0 low quality 1 medium quality 2 high quality, and 0 low quality by default
            wait_play_sound: Wait until the audio broadcast is finished; the default value is true
            speech_rate: speech speed, which is 0 by default, and the value range is -100 to 200
            voice_style: voice style, the default General style, and other styles refer to the development document language and speaker
            return: True or False
        """
        return "true" in self.SendData("metahumanSpeechCache", save_voice_folder, text, language, voice_name, quality, wait_play_sound, speech_rate, voice_style)

    def metahuman_insert_video(self, video_file_path: str, audio_file_path: str, wait_play_video: bool = True) -> bool:
        """
            数字人插入视频
            Digital people insert videos.

            video_file_path: 插入的视频文件路径
            audio_file_path: 插入的音频文件路径
            wait_play_video: 等待视频播放完毕，默认为 true等待
            return: True或者False

            video_file_path: the path of the inserted video file
            audio_file_path: the path of the inserted audio file
            wait_play_video: Wait until the video is finished; the default value is true
            return: True or False
        """
        return "true" in self.SendData("metahumanInsertVideo", video_file_path, audio_file_path, wait_play_video) 

    def replace_background(self, bg_file_path: str, replace_red: int = -1, replace_green: int = -1, replace_blue: int = -1, sim_value: int = 0) -> bool:
        """
            替换数字人背景
            Replace the background of digital people

            bg_file_path: 数字人背景 图片/视频 路径，默认不替换背景。仅替换绿幕背景的数字人模型
            replace_red: 数字人背景的三通道之一的 R通道色值。默认-1 自动提取
            replace_green: 数字人背景的三通道之一的 G通道色值。默认-1 自动提取
            replace_blue: 数字人背景的三通道之一的 B通道色值。默认-1 自动提取
            sim_value: 相似度。 默认为0，取值应当大于等于0
            return: True或者False

            bg_file_path: the background picture/video path of digital people, which does not replace the background by default. Digital human model only replacing green screen background
            replace_red: R channel color value of one of the three channels of digital human background. Default -1 automatic extraction
            replace_green: the G-channel color value of one of the three channels of digital human background. Default -1 automatic extraction
            replace_blue: B-channel color value of one of the three channels of digital human background. Default -1 automatic extraction
            sim_value: similarity. The default value is 0 and the value should be greater than or equal to 0
            return: True or False
        """
        return "true" in self.SendData("replaceBackground", bg_file_path, replace_red, replace_green, replace_blue, sim_value) 

    def show_speech_text(self, origin_y: int = 0, font_type: str = "Arial", font_size: int = 30, font_red: int = 128,font_green: int = 255, font_blue: int = 0, italic: bool = False,underline: bool = False) -> bool:
        """
            显示数字人说话的文本
            Displays the text spoken by a digital person

            origin_y, 第一个字显示的起始Y坐标点。 默认0 自适应高度
            font_type, 字体样式，支持操作系统已安装的字体。例如"Arial"、"微软雅黑"、"楷体"
            font_size, 字体的大小。默认30
            font_red, 字体颜色三通道之一的 R通道色值。默认128
            font_green, 字体颜色三通道之一的 G通道色值。默认255
            font_blue, 字体颜色三通道之一的 B通道色值。默认0
            italic, 是否斜体,默认false
            underline, 是否有下划线,默认false
            return: True或者False

            origin_y, the starting y coordinate point displayed by the first word. Default 0 Adaptive Height
            font_type, font style, supports fonts installed in the operating system. For example, "Arial", "Microsoft yahei" and "italics"
            font_size, the size of the font. Default 30
            font_red, the R channel color value of one of the three channels of font color. Default 128
            font_green, the color value of G channel, one of the three channels of font color. Default 255
            font_blue, the color value of B channel, which is one of the three channels of font color. Default 0
            italic, italic or not, the default is false
            underline, whether it is underlined or not, the default is false
            return: True or False
        """
        return "true" in self.SendData("showSpeechText", origin_y, font_type, font_size, font_red, font_green, font_blue,italic, underline) 

    def make_metahuman_video(self, save_video_folder: str, text: str, language: str, voice_name: str, bg_file_path: str, sim_value: float = 0, voice_style: str = "General", quality: int = 0, speech_rate: int = 0,) -> bool:
        """
            生成数字人短视频，此函数需要调用 initSpeechService 初始化语音服务
            To generate short video of digital people, this function needs to call initSpeechService to initialize voice service

            save_video_folder: 保存的视频目录
            text: 要转换语音的文本
            language: 语言，参考开发文档 语言和发音人
            voice_name: 发音人，参考开发文档 语言和发音人
            bg_file_path: 数字人背景 图片/视频 路径，扣除绿幕会自动获取绿幕的RGB值，null 则不替换背景。仅替换绿幕背景的数字人模型
            sim_value: 相似度，默认为0。此处参数用作绿幕扣除微调RBG值。取值应当大于等于0
            voice_style: 语音风格，默认General常规风格，其他风格参考开发文档 语言和发音人
            quality: 音质，0低品质  1中品质  2高品质， 默认为0低品质
            speech_rate:  语速，默认为0，取值范围 -100 至 200
            return: True或者False

            save_video_folder: the saved video directory
            text: the text to be converted into speech
            language: language, reference development document language and speaker
            voice_name: speaker, refer to the development document language and speaker
            bg_file_path: the background picture/video path of the digital person. If the green screen is subtracted, the RGB value of the green screen will be automatically obtained; if it is null, the background will not be replaced. Digital human model only replacing green screen background
            sim_value: similarity, which defaults to 0. Here, the parameter is used as the RBG value of green screen subtraction fine tuning. Value should be greater than or equal to 0
            voice_style: voice style, the default General style, and other styles refer to the development document language and speaker
            quality: sound quality, 0 low quality 1 medium quality 2 high quality, and 0 low quality by default
            speech_rate: speech speed, which is 0 by default, and the value range is -100 to 200
            return: True or False
        """
        return "true" in self.SendData("makeMetahumanVideo", save_video_folder, text, language, voice_name, bg_file_path, sim_value, voice_style, quality, speech_rate) 

    def init_speech_clone_service(self, api_key: str, voice_id: str) -> bool:
        """
            初始化数字人声音克隆服务(不支持win 7系统)  声音克隆服务卡密获取网站：elevenlabs.io
            Initialize the digital human voice cloning service (win 7 system is not supported). The voice cloning service card is obtained from the website: elevenlabs.io

            api_key: API密钥
            voice_id: 声音ID
            return: True或者False

            api_key: API key
            clone_val: Voice ID
            return: True or False
        """
        return "true" in self.SendData("initSpeechCloneService", api_key, voice_id) 

    def metahuman_speech_clone(self, save_audio_path: str, text: str, language: str = "zh-cn", wait_play_sound: bool = True) -> bool:
        """
            数字人使用克隆声音说话，此函数需要调用 initSpeechCloneService 初始化语音服务
            Generate digital human speech file (voice clone), and automatically generate MP3 file and lab file

            save_audio_path: 保存的发音文件路径, 这里是文件路径，不是目录！
            text: 要转换语音的文本
            language: 语言，中文：zh-cn，其他语言：other-languages
            wait_play_sound: 等待音频播报完毕，默认为 true等待
            return: True或者False

            save_audio_path: the path of the saved pronunciation file, here is the file path, not the directory!
            text: the text to be converted into speech
            language: language, Chinese: zh-cn, other languages: other-languages
            wait_play_sound: Wait until the audio broadcast is finished; the default value is true
            return: True or False
        """
        return "true" in self.SendData("metahumanSpeechClone", save_audio_path, text, language, wait_play_sound) 

    def make_metahuman_video_clone(self, save_video_folder: str, text: str, language: str = "zh-cn", bg_file_path: str = "", sim_value: int = 0) -> bool:
        """
            使用克隆声音生成数字人短视频，此函数需要调用 initSpeechCloneService 初始化语音服务
            Using cloned voice to generate short video of digital people, this function needs to call initSpeechCloneService to initialize voice service

            save_video_folder: 保存的视频和音频文件目录
            text: 要转换语音的文本
            language: 语言，语言，中文：zh-cn，其他语言：other-languages
            bg_file_path: 数字人背景 图片/视频 路径，扣除绿幕会自动获取绿幕的RGB值，null 则不替换背景。仅替换绿幕背景的数字人模型
            sim_value: 相似度，默认为0。此处参数用作绿幕扣除微调RBG值。取值应当大于等于0
            return: True或者False

            save_video_folder: directory of saved video and audio files
            text: the text to be converted into speech
            language: language, language, Chinese: zh-cn, other languages: other-languages
            bg_file_path: the background picture/video path of the digital person. If the green screen is subtracted, the RGB value of the green screen will be automatically obtained; if it is null,
                          the background will not be replaced. Digital human model only replacing green screen background
            sim_value: similarity, which defaults to 0. Here, the parameter is used as the RBG value of green screen subtraction fine tuning. Value should be greater than or equal to 0
            return: True or False
        """
        return "true" in self.SendData("makeMetahumanVideoClone", save_video_folder, text, language, bg_file_path, sim_value) 


    def make_metahuman_speech_file_clone(self, save_audio_path: str, text: str, language: str = "zh-cn") -> bool:
        """
            生成数字人说话文件(声音克隆)，生成MP3文件和 lab文件，提供给 metahumanSpeechByFile 和使用
            Generate digital human speech file (voice clone), MP3 file and lab file, and provide them to metahumanSpeechByFile and use it

            save_audio_path: 保存的发音文件路径。这里是路径，不是目录！
            text: 要转换语音的文本
            language: 语言，中文：zh-cn，其他语言：other-languages
            return: True或者False

            save_audio_path: the path of the saved pronunciation file. This is a path, not a directory!
            text: the text to be converted into speech
            language: language, Chinese: zh-cn, other languages: other-languages
            return: True or False
        """
        return "true" in self.SendData("makeMetahumanSpeechFileClone", save_audio_path, text, language) 

    def metahuman_speech_byFile(self, audio_path: str, wait_play_sound : bool = True) -> bool:
        """
            数字人说话文件缓存模式
            Cache mode of digital human speech file

            audio_path: 音频路径， 同名的 .lab文件需要和音频文件在同一目录下
            wait_play_sound: 是否等待播报完毕，默认为true 等待
            return: True或者False

            audio_path: Audio path. lab file with the same name needs to be in the same directory as the audio file
            wait_play_sound: whether to wait for the broadcast, the default value is true
            return: True or False
        """
        return "true" in self.SendData("metahumanSpeechByFile", audio_path, wait_play_sound) 

    def metahuman_speech_break(self) -> bool:
        """
            打断数字人说话，一般用作人机对话场景。metahumanSpeech和metahumanSpeechCache的 waitPlaySound 参数 设置为false时，此函数才有意义
            Interrupting a digital person's speech is generally used as a human-computer conversation scene. 
            This function only makes sense when the waitPlaySound parameter of metahumanSpeech and metahumanSpeechCache is set to false

            return: 返回true打断正在说话， 返回false 则为未说话状态
            return true to interrupt talking, return false to be silent
        """
        return "true" in self.SendData("metahumanSpeechBreak") 

    def make_metahuman_speech_file(self, save_audio_path: str, text: str, language: str = "zh-cn", voice_name: str = "", quality: int = 0, speech_rate: int = 0, voice_style: str = "General") -> bool:
        """
            生成数字人说话文件，生成MP3文件和 lab文件，提供给 metahumanSpeechByFile 和使用
            Generate digital human speech files, MP3 files and lab files, and provide them to metahumanSpeechByFile and use

            save_audio_path: 保存的音频文件路径，扩展为.MP3格式。同名的 .lab文件需要和音频文件在同一目录下
            text: 要转换语音的文本
            language: 语言，参考开发文档 语言和发音人
            voice_name: 发音人，参考开发文档 语言和发音人
            quality: 音质，0低品质  1中品质  2高品质， 默认为0低品质
            speech_rate: 语速，默认为0，取值范围 -100 至 200
            voice_style: 语音风格，默认General常规风格，其他风格参考开发文档 语言和发音人
            return: True或者False

            save_audio_path: the path of the saved audio file, expanded to .mp3 format. The. lab file with the same name needs to be in the same directory as the audio file
            text: the text to be converted into speech
            language: language, reference development document language and speaker
            voice_name: speaker, refer to the development document language and speaker
            quality: sound quality, 0 low quality 1 medium quality 2 high quality, and 0 low quality by default
            speech_rate: speech speed, which is 0 by default, and the value range is -100 to 200
            voice_style: voice style, the default General style, and other styles refer to the development document language and speaker
            return: True or False
        """
        return "true" in self.SendData("makeMetahumanSpeechFile", save_audio_path, text, language, voice_name, quality, speech_rate, voice_style) 

    def switch_action(self, call_apiKey: str, action_video_or_image: str, user_simValue: bool) -> bool:
        """
            切换新的人物形象动作，此函数无需训练数字人模型，直接切换各种人物形象动作和场景
            Switch new character movements. This function can directly switch various character movements and scenes without training digital human models

            call_apiKey: 调用函数的密钥
            action_video_or_image: 闭嘴的人物视频或者图片
            user_simValue: 是否使用近似值加速(影响精度)，默认不加速
            return: True或者False

            call_apiKey: String, the key of the calling function
            action_video_or_image: a string, shut-up character video or picture
            return: True or False
        """
        return "true" in self.SendData("switchAction", call_apiKey, action_video_or_image, user_simValue) 

    def train_human_model(self, call_apiKey: str, train_video_or_image: str, src_metahuman_model_path: str, save_human_model_folder: str) -> bool:
        """
            训练数字人，训练时长为10-30分钟
            Train digital people for 10-30 minutes

            call_apiKey: 调用函数的密钥
            train_video_or_image: 闭嘴的人物视频或者图片 素材
            src_metahuman_model_path: 预训练数字人模型路径
            save_human_model_folder: 保存训练完成的模型目录
            return: True或者False

            call_apiKey: the key of the calling function
            train_video_or_image: shut up the character video or picture material
            src_metahuman_model_path: the path of pre-training digital human model
            save_human_model_folder: save the model directory after training
            return: True or False
        """
        return "true" in self.SendData("trainHumanModel", call_apiKey, train_video_or_image, src_metahuman_model_path, save_human_model_folder) 

    def make_metahuman_video_by_file(self, audio_path: str, bg_file_path: str, sim_value: str = 0) -> bool:
        """
            通过音频文件生成数字人短视频， 此函数依赖 initMetahuman 函数运行，否则程序会崩溃
            Switch new character movements. This function can directly switch various character movements and scenes without training digital human models

            audio_path: 字符串型，音频路径， 同名的 .lab文件需要和音频文件在同一目录下
            bg_file_path: 字符串型，数字人背景 图片/视频 路径，扣除绿幕会自动获取绿幕的RGB值，null 则不替换背景。仅替换绿幕背景的数字人模型
            sim_value: 整型，相似度，默认为0。此处参数用作绿幕扣除微调RBG值。取值应当大于等于0
            return: True or False

            audio_path: String type, audio path, and. lab file with the same name need to be in the same directory as the audio file
            bg_file_path: String type, background picture/video path of digital person. If the green screen is subtracted, the RGB value of the green screen will be automatically obtained,
                           and null will not replace the background. Digital human model only replacing green screen background
            sim_value: Integer, similarity, default is 0. Here, the parameter is used as the RBG value of green screen subtraction fine tuning
                       Value should be greater than or equal to 0
            return: True or False
        """
        return "true" in self.SendData("makeMetahumanVideoByFile", audio_path, bg_file_path, sim_value) 

    def get_switch_action_state(self) -> bool:
        """
            获取切换人物形象动作状态
            Obtain that action state of switching the character image
   
            return: True或者False

            return: True or False
        """
        return "true" in self.SendData("getSwitchActionState") 

    def make_clone_audio(self, clone_server_ip: str, save_audio_path: str, refer_audio_path: str, refer_text: str, clone_text: str, speed_factor: float) -> bool:
        """
            克隆声音，需要部署服务端
            To clone sound, you need to deploy the server.

            clone_server_ip: 克隆声音服务端IP
            save_audio_path: 保存克隆声音的路径
            refer_audio_path: 参考音频路径，10-40秒，推荐25秒左右的参考音频
            refer_text: 参考音频对应的文本
            clone_text: 要克隆的文本
            speed_factor: 语速（0.5为半速，1.0为正常速度，1.5为1.5倍速，以此类推）。默认为1.0 正常语速
            return: True或者False

            clone_server_ip: clone voice server IP
            save_audio_path: the path to save the cloned sound
            refer_audio_path: Reference audio path, 10-40 seconds, 25 seconds is recommended
            refer_text: the text corresponding to the reference audio
            clone_text: the text to be cloned
            speed_factor: speech speed (0.5 is half speed, 1.0 is normal speed, 1.5 is 1.5 times speed, and so on). The default is 1.0 normal speech speed
            return: True or False
        """
        return "true" in self.SendData("makeCloneAudio", clone_server_ip ,save_audio_path, refer_audio_path, refer_text, clone_text, speed_factor)


    def make_clone_lab(self, lab_server_ip: str, audio_path: str) -> bool:
        """
            生成lab文件，需要部署服务端
            To generate lab files, you need to deploy the server.

            lab_server_ip: 保存的发音文件路径。这里是路径，不是目录！
            audio_path: 要转换语音的文本
            return: True或者False

            lab_server_ip: saved pronunciation file path. This is a path, not a directory
            audio_path: the text to be converted into speech
            return: True or False
        """
        return "true" in self.SendData("makeCloneLab", lab_server_ip, audio_path) 

    def clone_audio_to_text(self, lab_server_ip: str, audio_path: str) -> bool:
        """
            语音识别，需要部署服务端
            Speech recognition requires the deployment of the server.

            lab_server_ip: lab服务端IP
            audio_path: 音频文件
            return: 失败返回None, 成功返回识别到的内容

            lab_server_ip: lab server IP
            audio_path: audio file
            return: None is returned in case of failure, and the recognized content is returned successfully
        """
        return "true" in self.SendData("cloneAudioToText", lab_server_ip, audio_path)
    
    def text_to_audio_and_lab_file(self, save_audio_path: str, text: str, language: str = "zh-cn", voice_name: str = "", quality: int = 0, speech_rate: int = 0, voice_style: str = "General") -> bool:
        """
            文本合成语音保存到本地音频文件
            Text synthesis speech is saved to a local audio file

            save_audio_path: 保存的音频文件路径，扩展为.MP3格式。同名的 .lab文件需要和音频文件在同一目录下
            text: 要转换语音的文本
            language: 语言，参考开发文档 语言和发音人
            voice_name: 发音人，参考开发文档 语言和发音人
            quality: 音质，0低品质  1中品质  2高品质， 默认为0低品质
            speech_rate: 语速，默认为0，取值范围 -100 至 200
            voice_style: 语音风格，默认General常规风格，其他风格参考开发文档 语言和发音人
            return: True或者False

            save_audio_path: the path of the saved audio file, expanded to .mp3 format. The. lab file with the same name needs to be in the same directory as the audio file
            text: the text to be converted into speech
            language: language, reference development document language and speaker
            voice_name: speaker, refer to the development document language and speaker
            quality: sound quality, 0 low quality 1 medium quality 2 high quality, and 0 low quality by default
            speech_rate: speech speed, which is 0 by default, and the value range is -100 to 200
            voice_style: voice style, the default General style, and other styles refer to the development document language and speaker
            return: True or False
        """
        return "true" in self.SendData("textToAudioAndLabFile", save_audio_path, text, language, voice_name, quality, speech_rate, voice_style)
    

    def switch_clone_audio_model(self, clone_server_ip: str, gpt_weights_path: str, sovits_weights_path: str) -> bool:
        """
            切换声音克隆模型，需要部署服务端(切换到与原模型无关音色的模型，切记更换参考音频和文本)
            To switch the voice cloning model, you need to deploy the server

            clone_server_ip: 克隆声音服务端
            gpt_weights_path: gpt 模型权重路径。指克隆服务所在的电脑/服务器 路径
            sovits_weights_path: sovits 模型权重路径。指克隆服务所在的电脑/服务器 路径
            return: 成功返回True，失败返回False

            clone_server_ip: clone the sound server
            gpt _ weights _ path: the weight path of GPT model. Refers to the computer/server path where the cloning service is located
            sovits _ weights _ path: the weight path of Sovits model. Refers to the computer/server path where the cloning service is located
            return: Returns True on success and False on failure
        """
        return "true" in self.SendData("switchCloneAudioModel", clone_server_ip, gpt_weights_path, sovits_weights_path)
    

    def restart_clone_audio_server(self, clone_server_ip: str) -> bool:
        """
            重启声音克隆服务，需要部署服务端
            To restart the voice cloning service, the server needs to be deployed

            clone_server_ip: 克隆声音服务端
            return: 成功返回True，失败返回False

            clone_server_ip: clone the sound server
            return: Returns True on success and False on failure
        """
        return "true" in self.SendData("restartCloneAudioServer", clone_server_ip)


    def play_audio(self, audio_path: str, is_wait: bool = True) -> bool:
        """
            播报音频文件 不需要lab文件
            Broadcast audio file

            audio_path: 音频文件路径
            is_wait: 是否等待，为True时，等待播放完毕
            return: 成功返回True，失败返回False

            audio_path: audio file path
            is_wait: whether to wait, if True, wait for the playback to finish
            return: Returns True on success and False on failure
        """
        return "true" in self.SendData("playAudio", audio_path, is_wait)
    

    def play_media(self, video_path: str, video_sacle: float = 1.0, isLoop_play: bool = False, enable_random_param: bool = False, is_wait: bool = True) -> bool:
        """
            播报视频文件 (多个视频切换播放 视频和音频编码必须一致)
            Broadcast video frequency files (video and audio coding must be consistent when playing multiple videos)

            video_path: 视频文件路径
            video_sacle: 视频缩放（0.5缩小一半，1.0为原始大小）
            isLoop_play: 是否循环播放
            enable_random_param: 是否启用随机去重参数
            is_wait: 是否等待播报完毕。值为false时，不等待播放结束。未播报结束前再次调用此函数 会终止前面的播报内容
            return: 成功返回True，失败返回False

            video_path: video file path
            video_sacle: video scaling (0.5 is reduced by half, 1.0 is the original size)
            isLoop_play: whether to loop or not
            enable_random_param: whether to enable the random deduplication parameter
            is_wait: whether to wait for the broadcast to finish. When the value is false, do not wait for the end of playback. Calling this function again before the broadcast ends will terminate the previous broadcast content
            return: Returns True on success and False on failure
        """

        return "true" in self.SendData("playMedia", video_path, video_sacle, isLoop_play, enable_random_param, is_wait)
    

    def extract_audio(self, video_path: str) -> bool:
        """
            复制视频中的音频为mp3文件
            Copy the audio in the video as an mp3 file.

            video_path: 视频文件路径 音频文件保存为mp3和视频在一个目录
            return: 成功返回True，失败返回False

            video_path: video file path. Audio files are saved as mp3 and videos are in one directory
            return: Returns True on success and False on failure
        """
        
        return "true" in self.SendData("extractAudio", video_path)


    def set_media_volume_scale(self, volume_scale: float) -> bool:
        """
            调节 playMedia 音量大小(底层用的内存共享，支持多进程控制)
            Adjust the volume of playMedia (memory sharing for the bottom layer, supporting multi-process control)

            volume_scale: 浮点型，音量缩放（0.5调低一半，1.0为原始音量大小）。默认为原始大小
            return: 总是返回True

            volume_scale: floating-point, volume scaling (0.5 is reduced by half, 1.0 is the original volume). Default is the original size
            return: always returns True
        """
        
        return "true" in self.SendData("setMediaVolumeScale", volume_scale)


    def metahuman_speech_by_file_ex(self, audio_path: str, enable_random_param: bool , wait_play_sound: bool = True) -> bool:
        """
            数字人说话文件缓存模式(Ex)，不能与 PlayAudioEx 同步执行
            Digital human speech file cache mode (Ex) cannot be synchronized with PlayAudioEx

            audio_path: 音频路径， 同名的 .lab文件需要和音频文件在同一目录下。若.lab文件不存在，则自动生成.lab文件。生成.lab文件产生的费用，请联系管理员
            enable_random_param: 是否启用随机去重参数
            wait_play_sound : 是否等待播报完毕，默认为True 等待。为False时 多次调用此函数会添加到队列按顺序播报
            return: 成功返回True，失败返回False

            audio_path: the audio path. The. lab file with the same name needs to be in the same directory as the audio file. If the. lab file does not exist, the. lab file is automatically generated. Please contact the administrator for the expenses incurred by generating the. lab file
            enable_random_param: whether to enable the random deduplication parameter
            wait_play_sound: whether to wait for the broadcast, the default value is True. When False, calling this function multiple times will be added to the queue to broadcast in sequence
            return: Returns True on success and False on failure

        """
        
        return "true" in self.SendData("metahumanSpeechByFileEx", audio_path, enable_random_param, wait_play_sound)
    

    def play_audio_ex(self, audio_path, enable_random_param, is_wait) -> bool:
        """
            播报音频文件(EX)，不能与 metahumanSpeechByFileEx 同步执行
            Broadcast audio file (EX), which can't be executed synchronously with metahumanSpeechByFileEx

            audio_path: 音频文件路径
            enable_random_param: 是否启用随机去重参数
            is_wait : 是否等待。为true时,等待播放完毕
            return: 总是返回true，函数仅添加播放音频文件到队列不处理返回

            audio_path: the audio path. The. lab file with the same name needs to be in the same directory as the audio file. If the. lab file does not exist, the. lab file is automatically generated. Please contact the administrator for the expenses incurred by generating the. lab file
            enable_random_param: whether to enable the random deduplication parameter
            wait_play_sound: whether to wait for the broadcast, the default value is True. When False, calling this function multiple times will be added to the queue to broadcast in sequence
            return: Returns True on success and False on failure

        """
        
        return "true" in self.SendData("playAudioEx", audio_path, enable_random_param, is_wait)


import websockets
import asyncio
import threading
import json,traceback
from loguru import logger
from . import GlobalVariable


class WebSocketServerThread(threading.Thread):
    """
        运行 WebSocket 服务器的线程
        Thread running WebSocket server
    """

    def __init__(self, ip, port, is_print):
        super().__init__()
        self._loop = None
        self._server = None
        self._running = False
        self._ip = ip
        self._port = port
        self._is_print = is_print

    def debug(self, msg):
        if self._is_print == "DEBUG":
            logger.debug(msg)


    async def handle_connection(self, websocket):
        """
            处理客户端连接的协程
            Ctrip handling client connection
        """
        try:

            self.debug(f"Websocket Client connection at: <-<- {websocket.remote_address}")
            async for message in websocket:
                Msg = json.loads(message)    
                GlobalVariable.SocketServerClients[Msg["Data"]["Token"]] = websocket 

                self.debug(f"Websocket Receive Msg: <-<- {websocket.remote_address}")

                GlobalVariable.WebsocketReceiveMessage.put(Msg)
                await websocket.send(f"Server received: {message}")
        except Exception as e:
            pass

        finally:
            del GlobalVariable.SocketServerClients[Msg["Data"]["Token"]]   
            self.debug(f"Websocket Disconnect: <-<- {websocket.remote_address}")


    async def server_main(self):
        """
            WebSocket 服务器主逻辑
            WebSocket server main logic
        """
        self._running = True
        try:
            self._server = await websockets.serve(self.handle_connection, self._ip, self._port)
            print(f"WebSocket Server Stared on ws://{self._ip}:{self._port}")
            while self._running:
                if not GlobalVariable.WebsocketCallBack.empty():
                    item = GlobalVariable.WebsocketCallBack.get()
                    for token, client in GlobalVariable.SocketServerClients.items():
                        if token == item['Token']:
                            await client.send(str(item))
                await asyncio.sleep(1)
        except Exception as e:
            logger.error(traceback.format_exc())


    def run(self):
        """
            线程入口
            Thread entry
        """
        self._loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self._loop)
        try:
            self._loop.run_until_complete(self.server_main())
        except asyncio.CancelledError:
            pass
        finally:
            self._loop.close()

    def stop(self):
        """
            停止服务器
            Stop the server
        """
        if self._running:
            self._running = False
            if self._server:
                # 停止服务器
                self._loop.call_soon_threadsafe(self._server.close)


    def is_running(self):
        return self._running
    

class WebSocketServerUse:

    def get_websocket_data(self) -> dict:
        """
            获取WebSocket传递过来的数据
            Get the data passed by WebSocket

            return: 成功返回字典，失败返回空字典
            return: The dictionary was returned successfully, and an empty dictionary was returned if it failed
        """

        if not GlobalVariable.WebsocketReceiveMessage.empty():
            item = GlobalVariable.WebsocketReceiveMessage.get()
            return item
        return {}
    

    def send_websocket_data(self, data) -> bool:
        """
            发送WebSocket数据给客户端需要指定Token
            Sending WebSocket data to the client needs to specify Token

            data: 需要发送到客户端的数据
            data: the data that needs to be sent to the client

            return: 成功返回字典，失败抛错
            return: successfully returned to the dictionary, failed to throw a mistake
        """
        if "Token" not in data:
            raise KeyError("Token is not in data")

        if "Data" not in data:
            raise KeyError("Data is not in data")


        GlobalVariable.WebsocketCallBack.put(data)
        return True
    

    


import base64
from urllib import request as request_lib, parse
import json

class VerificationCodeOperation:
    """
        验证码操作
        Verification code operation
    """
    def get_captcha(self, file_path: str, username: str, password: str, soft_id: str, code_type: str, len_min: str = '0') -> dict:
        """
            识别验证码
            Identification verification code

            file_path: 图片文件路径
            username: 用户名
            password: 密码
            soft_id: 软件ID
            code_type: 图片类型 参考https://www.chaojiying.com/price.html
            len_min: 最小位数 默认0为不启用,图片类型为可变位长时可启用这个参数
            return: JSON
                err_no,(数值) 返回代码  为0 表示正常，错误代码 参考https://www.chaojiying.com/api-23.html
                err_str,(字符串) 中文描述的返回信息
                pic_id,(字符串) 图片标识号，或图片id号
                pic_str,(字符串) 识别出的结果
                md5,(字符串) md5校验值,用来校验此条数据返回是否真实有效

            file_path: the path of the picture file
            username: user name
            password: password
            soft_id: software id
            code_type: the picture type refers to https://www.chaojiying.com/price.html
            len_min: The minimum number of digits is not enabled by default, and this parameter can be enabled when the picture type is variable bit length
            return: JSON
                err_no, (numerical value) The return code is 0, which means normal, and the error code refers to https://www.chaojiying.com/api-23.html
                err_str, (string) the return information described in Chinese
                pic_id, (string) picture identification number, or picture ID number
                pic_str, (string) the result of recognition
                md5, (string) md5 check value, which is used to check whether this data return is true and valid
        """
        file = open(file_path, mode="rb")
        file_data = file.read()
        file_base64 = base64.b64encode(file_data)
        file.close()
        url = "http://upload.chaojiying.net/Upload/Processing.php"
        data = {
            'user': username,
            'pass': password,
            'softid': soft_id,
            'codetype': code_type,
            'len_min': len_min,
            'file_base64': file_base64
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:24.0) Gecko/20100101 Firefox/24.0',
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        parse_data = parse.urlencode(data).encode('utf8')
        req = request_lib.Request(url, parse_data, headers)
        response = request_lib.urlopen(req)
        result = response.read().decode()
        return json.loads(result)

    def error_captcha(self, username: str, password: str, soft_id: str, pic_id: str) -> dict:
        """
            识别报错返分
            Identify and report errors and return points

            username: 用户名
            password: 密码
            soft_id: 软件ID
            pic_id: 图片ID 对应 getCaptcha返回值的pic_id 字段
            return: JSON
                err_no,(数值) 返回代码
                err_str,(字符串) 中文描述的返回信息

            username: user name
            password: password
            soft_id: software id
            pic_id: the picture id corresponds to the pic_id field of getCaptcha return value
            return: JSON
                err_no, (numeric) return code
                err_str, (string) the return information described in Chinese
        """
        url = "http://upload.chaojiying.net/Upload/ReportError.php"
        data = {
            'user': username,
            'pass': password,
            'softid': soft_id,
            'id': pic_id,
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:24.0) Gecko/20100101 Firefox/24.0',
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        parse_data = parse.urlencode(data).encode('utf8')
        req = request_lib.Request(url, parse_data, headers)
        response = request_lib.urlopen(req)
        result = response.read().decode()
        return json.loads(result)

    def score_captcha(self, username: str, password: str) -> dict:
        """
            查询验证码剩余题分
            Query the remaining questions of verification code

            username: 用户名
            password: 密码
            return: JSON
                err_no,(数值) 返回代码
                err_str,(字符串) 中文描述的返回信息
                tifen,(数值) 题分
                tifen_lock,(数值) 锁定题分

            username: user name
            password: password
            return: JSON
                err_no, (numeric) return code
                err_str, (string) the return information described in Chinese
                tifen, (numerical) score
                tifen_lock, (numerical value) locks the score
        """
        url = "http://upload.chaojiying.net/Upload/GetScore.php"
        data = {
            'user': username,
            'pass': password,
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:24.0) Gecko/20100101 Firefox/24.0',
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        parse_data = parse.urlencode(data).encode('utf8')
        req = request_lib.Request(url, parse_data, headers)
        response = request_lib.urlopen(req)
        result = response.read().decode()
        return json.loads(result)




import re

class PopUpWindow:
    """
        点击alert警告框
        Click the alert warning box.
    """
    
    def click_alert(self, accept: bool, prompt_text: str = "") -> bool:
        """
            点击警告框
            Click the warning box

            accept: 确认或取消
            prompt_text: 可选参数，输入的警告框文本
            return: 布尔值

            accept: Confirm or cancel
            prompt_text: Optional parameter, warning box text entered.
            return: bool
        """
        return "true" in self.SendData("clickAlert", accept, prompt_text) 

    def get_alert_text(self) -> str:
        """
            获取警告框文本
            Get warning box text

            return: 警告框文本字符串
            return: Return: warning box text string
        """
        response = self.SendData("getAlertText")
        if response == "null":
            return None
        return response